# Copyright (c) 2022 PaddlePaddle Authors. All Rights Reserved.
# Copyright 2021 The Open AI Team Authors and The HuggingFace Inc. team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from functools import partial
from typing import Any, Optional, Tuple, Union

import paddle
import paddle.distributed as dist
import paddle.nn as nn
import paddle.nn.functional as F

from ...utils.initializer import normal_
from ...utils.log import logger
from .. import PretrainedModel
from ..clip.modeling import CLIPVisionTransformer as ErnieViLVisionTransformer
from ..clip.modeling import clip_loss
from ..configuration_utils import PretrainedConfig
from ..ernie.modeling import ErnieModel
from ..guided_diffusion_utils import (
    DiscoDiffusionMixin,
    create_gaussian_diffusion,
    create_secondary_model,
    create_unet_model,
)
from ..model_outputs import (
    BaseModelOutputWithPooling,
    BaseModelOutputWithPoolingAndCrossAttentions,
    ModelOutput,
)
from ..utils import resolve_cache_dir
from .configuration import ErnieViLConfig, ErnieViLTextConfig, ErnieViLVisionConfig

__all__ = [
    "ErnieViLModel",
    "ErnieViLTextModel",
    "ErnieViLVisionModel",
    "ErnieViLPretrainedModel",
    "ErnieViLForImageGeneration",
]

ERNIE_VIL_PRETRAINED_MODEL_ARCHIVE_LIST = [
    # vit model
    "PaddlePaddle/ernie_vil-2.0-base-zh",
    "PaddlePaddle/disco_diffusion_ernie_vil-2.0-base-zh",
]


def quick_gelu(x):
    return x * F.sigmoid(1.702 * x)


F.quick_gelu = quick_gelu


@dataclass
class ErnieViLOutput(ModelOutput):
    """
    Args:
        loss: (`paddle.Tensor` of shape `(1,)`, *optional*, returned when `return_loss` is `True`):
            Contrastive loss for image-text similarity.
        logits_per_image: (`paddle.Tensor` of shape `(image_batch_size, text_batch_size)`):
            The scaled dot product scores between `image_embeds` and `text_embeds`. This represents the image-text
            similarity scores.
        logits_per_text: (`paddle.Tensor` of shape `(text_batch_size, image_batch_size)`):
            The scaled dot product scores between `text_embeds` and `image_embeds`. This represents the text-image
            similarity scores.
        text_embeds: (`paddle.Tensor` of shape `(batch_size, output_dim`):
            The text embeddings obtained by applying the projection layer to the pooled output of [`ErnieModel`].
        image_embeds: (`paddle.Tensor` of shape `(batch_size, output_dim`):
            The image embeddings obtained by applying the projection layer to the pooled output of [`ErnieViLVisionTransformer`].
        text_model_output: (:class:`~paddlenlp.transformers.model_outputs.BaseModelOutputWithPoolingAndCrossAttentions`):
            The output of the [`ErnieModel`].
        vision_model_output: (:class:`~paddlenlp.transformers.model_outputs.BaseModelOutputWithPooling`):
            The output of the [`VisionTransformer`].
    """

    loss: Optional[paddle.Tensor] = None
    logits_per_image: paddle.Tensor = None
    logits_per_text: paddle.Tensor = None
    text_embeds: paddle.Tensor = None
    image_embeds: paddle.Tensor = None
    text_model_output: BaseModelOutputWithPoolingAndCrossAttentions = None
    vision_model_output: BaseModelOutputWithPooling = None

    def to_tuple(self) -> Tuple[Any]:
        return tuple(
            self[k] if k not in ["text_model_output", "vision_model_output"] else getattr(self, k).to_tuple()
            for k in self.keys()
        )


class ErnieViLPretrainedModel(PretrainedModel):
    """
    An abstract class for pretrained ErnieViL models. It provides ErnieViL related
    `model_config_file`, `pretrained_init_configuration`, `resource_files_names`,
    `pretrained_resource_files_map`, `base_model_prefix` for downloading and
    loading pretrained models.
    See :class:`~paddlenlp.transformers.model_utils.PretrainedModel` for more details.
    """

    config_class = ErnieViLConfig
    base_model_prefix = "ernie_vil"
    supports_gradient_checkpointing = True

    def init_weights(self):
        """
        A method executed at the end of each Transformer model initialization, to execute code that needs the model's
        modules properly initialized (such as weight initialization).
        """
        self.apply(self._init_weights)

    def _set_gradient_checkpointing(self, module, value=False):
        if isinstance(module, nn.TransformerEncoder):
            module.enable_recompute = value

    def gradient_checkpointing_enable(self):
        """
        Activates gradient checkpointing for the current model.

        Note that in other frameworks this feature can be referred to as "activation checkpointing" or "checkpoint
        activations".
        """
        if not self.supports_gradient_checkpointing:
            raise ValueError(f"{self.__class__.__name__} does not support gradient checkpointing.")
        self.apply(partial(self._set_gradient_checkpointing, value=True))

    def gradient_checkpointing_disable(self):
        """
        Deactivates gradient checkpointing for the current model.

        Note that in other frameworks this feature can be referred to as "activation checkpointing" or "checkpoint
        activations".
        """
        if self.supports_gradient_checkpointing:
            self.apply(partial(self._set_gradient_checkpointing, value=False))

    def _init_weights(self, layer):
        """Initialize the weights"""
        if isinstance(layer, ErnieViLVisionTransformer):
            # find nn.LayerNorm
            for sub_layer in layer.sublayers():
                if isinstance(sub_layer, nn.LayerNorm):
                    sub_layer._epsilon = layer.config.layer_norm_eps

        elif isinstance(layer, ErnieModel):
            # find nn.LayerNorm
            for sub_layer in layer.sublayers():
                if isinstance(sub_layer, nn.LayerNorm):
                    sub_layer._epsilon = layer.config.layer_norm_eps
                elif isinstance(layer, (nn.Linear, nn.Embedding)):
                    normal_(layer.weight, mean=0.0, std=layer.config.initializer_range)

    @classmethod
    def from_pretrained_v2(cls, pretrained_model_name_or_path, from_hf_hub: bool = False, *args, **kwargs):
        load_state_as_np = kwargs.pop("load_state_as_np", False)
        config = kwargs.pop("config", None)
        force_download = kwargs.pop("force_download", False)
        ignore_mismatched_sizes = kwargs.pop("ignore_mismatched_sizes", None)
        dtype = kwargs.pop("dtype", None)
        cache_dir = kwargs.pop("cache_dir", None)

        cache_dir = resolve_cache_dir(pretrained_model_name_or_path, from_hf_hub, cache_dir)

        model_kwargs = kwargs
        # 1. get the PretrainedConfig to init model
        if not isinstance(config, PretrainedConfig):
            config_path = config if config is not None else pretrained_model_name_or_path
            config, model_kwargs = cls.config_class.from_pretrained(
                config_path,
                cache_dir=cache_dir,
                return_unused_kwargs=True,
                force_download=force_download,
                from_hf_hub=from_hf_hub,
                **kwargs,
            )
        # Attention! we donot save this config.json
        # config.save_pretrained(cache_dir)

        # 2. init the model
        init_args = config["init_args"] or ()
        model = cls(config, *init_args, **model_kwargs)

        # 3. resolve model_weight file
        model_weight_file = cls._resolve_model_file_path(
            pretrained_model_name_or_path, cache_dir=cache_dir, from_hf_hub=from_hf_hub
        )

        # 4. loading the state dict
        model_state_dict = paddle.load(model_weight_file, return_numpy=load_state_as_np)

        loaded_state_dict_keys = list(model_state_dict.keys())
        # TODO(wj-Mcat): load shard checkpoint weight file, refer to: https://github.com/huggingface/transformers/pull/16343
        model, missing_keys, unexpected_keys, mismatched_keys = cls._load_pretrained_model(
            model=model,
            state_dict=model_state_dict,
            loaded_keys=loaded_state_dict_keys,
            ignore_mismatched_sizes=ignore_mismatched_sizes,
            dtype=dtype,
        )

        if len(unexpected_keys) > 0:
            logger.warning(
                f"Some weights of the model checkpoint at {pretrained_model_name_or_path} were not used when"
                f" initializing {model.__class__.__name__}: {unexpected_keys}\n- This IS expected if you are"
                f" initializing {model.__class__.__name__} from the checkpoint of a model trained on another task or"
                " with another architecture (e.g. initializing a BertForSequenceClassification model from a"
                " BertForPreTraining model).\n- This IS NOT expected if you are initializing"
                f" {model.__class__.__name__} from the checkpoint of a model that you expect to be exactly identical"
                " (initializing a BertForSequenceClassification model from a BertForSequenceClassification model)."
            )
        else:
            logger.info(f"All model checkpoint weights were used when initializing {model.__class__.__name__}.\n")

        if len(missing_keys) > 0:
            logger.warning(
                f"Some weights of {model.__class__.__name__} were not initialized from the model checkpoint at"
                f" {pretrained_model_name_or_path} and are newly initialized: {missing_keys}\nYou should probably"
                " TRAIN this model on a down-stream task to be able to use it for predictions and inference."
            )
        elif len(mismatched_keys) == 0:
            logger.info(
                f"All the weights of {model.__class__.__name__} were initialized from the model checkpoint at"
                f" {pretrained_model_name_or_path}.\nIf your task is similar to the task the model of the checkpoint"
                f" was trained on, you can already use {model.__class__.__name__} for predictions without further"
                " training."
            )
        if len(mismatched_keys) > 0:
            mismatched_warning = "\n".join(
                [
                    f"- {key}: found shape {shape1} in the checkpoint and {shape2} in the model instantiated"
                    for key, shape1, shape2 in mismatched_keys
                ]
            )
            logger.warning(
                f"Some weights of {model.__class__.__name__} were not initialized from the model checkpoint at"
                f" {pretrained_model_name_or_path} and are newly initialized because the shapes did not"
                f" match:\n{mismatched_warning}\nYou should probably TRAIN this model on a down-stream task to be able"
                " to use it for predictions and inference."
            )
        if paddle.in_dynamic_mode():
            return model

        return model, model_state_dict


class ErnieViLModel(ErnieViLPretrainedModel):
    r"""
    The bare ErnieViL Model outputting logits_per_image and logits_per_text.
    This model inherits from :class:`~paddlenlp.transformers.model_utils.PretrainedModel`.
    Refer to the superclass documentation for the generic methods.
    This model is also a Paddle `paddle.nn.Layer <https://www.paddlepaddle.org.cn/documentation
    /docs/en/api/paddle/fluid/dygraph/layers/Layer_en.html>`__ subclass. Use it as a regular Paddle Layer
    and refer to the Paddle documentation for all matter related to general usage and behavior.

    Args:
        config (:class:`ErnieViLConfig`):
            An instance of ErnieViLConfig used to construct ErnieViLModel.
    """
    config_class = ErnieViLConfig

    def __init__(self, config: ErnieViLConfig):
        super().__init__(config)

        if not isinstance(config.text_config, ErnieViLTextConfig):
            raise ValueError(
                "config.text_config is expected to be of type ErnieViLTextConfig but is of type"
                f" {type(config.text_config)}."
            )

        if not isinstance(config.vision_config, ErnieViLVisionConfig):
            raise ValueError(
                "config.vision_config is expected to be of type ErnieViLVisionConfig but is of type"
                f" {type(config.vision_config)}."
            )

        text_config = config.text_config
        vision_config = config.vision_config

        self.text_model = ErnieModel(text_config)

        self.vision_model = ErnieViLVisionTransformer(vision_config)

        self.temperature = self.create_parameter(
            shape=(1,),
            default_initializer=nn.initializer.Constant(config.logit_scale_init_value),
            dtype=paddle.get_default_dtype(),
        )

        self.init_weights()

    def get_image_features(
        self,
        pixel_values: Optional[paddle.Tensor] = None,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        return_dict: Optional[bool] = None,
    ) -> paddle.Tensor:
        r"""
        Args:
            pixel_values (`paddle.Tensor` of shape `(batch_size, num_channels, height, width)`):
                Pixel values. Padding will be ignored by default should you provide it. Pixel values can be obtained using
                [`ErnieViLFeatureExtractor`]. See [`ErnieViLFeatureExtractor.__call__`] for details.
            output_attentions (`bool`, *optional*):
                Whether or not to return the attentions tensors of all attention layers. See `attentions` under returned
                tensors for more detail.
            output_hidden_states (`bool`, *optional*):
                Whether or not to return the hidden states of all layers. See `hidden_states` under returned tensors for
                more detail.
            return_dict (`bool`, *optional*):
                Whether or not to return a [`BaseModelOutputWithPooling`] instead of a plain tuple.

        Returns:
            image_features (`paddle.Tensor` of shape `(batch_size, output_dim`): The image embeddings obtained by
            applying the projection layer to the pooled output of [`ErnieViLVisionModel`].

        Examples:
            .. code-block::

                import requests
                from PIL import Image
                from paddlenlp.transformers import ErnieViLProcessor, ErnieViLModel

                model = ErnieViLModel.from_pretrained("PaddlePaddle/ernie_vil-2.0-base-zh")
                processor = ErnieViLProcessor.from_pretrained("PaddlePaddle/ernie_vil-2.0-base-zh")

                url = "http://images.cocodataset.org/val2017/000000039769.jpg"
                image = Image.open(requests.get(url, stream=True).raw)
                inputs = processor(images=image, return_tensors="pd")
                image_features = model.get_image_features(**inputs)

        """
        output_attentions = output_attentions if output_attentions is not None else self.config.output_attentions
        output_hidden_states = (
            output_hidden_states if output_hidden_states is not None else self.config.output_hidden_states
        )
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        vision_outputs = self.vision_model(
            pixel_values,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict,
        )
        image_features = vision_outputs[1]
        return image_features

    def get_text_features(
        self,
        input_ids,
        attention_mask: Optional[paddle.Tensor] = None,
        position_ids: Optional[paddle.Tensor] = None,
        token_type_ids: Optional[paddle.Tensor] = None,
        task_type_ids: Optional[paddle.Tensor] = None,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        return_dict: Optional[bool] = None,
    ):
        r"""
        Args:
            input_ids (`paddle.Tensor` of shape `(batch_size, sequence_length)`):
                Indices of input sequence tokens in the vocabulary. Padding will be ignored by default should you provide
                it.
                Indices can be obtained using [`ErnieViLTokenizer`].
            attention_mask (`paddle.Tensor` of shape `(batch_size, sequence_length)`, *optional*):
                Mask to avoid performing attention on padding token indices. Mask values selected in `[0, 1]`:
                - 1 for tokens that are **not masked**,
                - 0 for tokens that are **masked**.
            position_ids (`paddle.Tensor` of shape `(batch_size, sequence_length)`, *optional*):
                Indices of positions of each input sequence tokens in the position embeddings. Selected in the range `[0,
                config.max_position_embeddings - 1]`.
            token_type_ids (`paddle.Tensor` of shape `(batch_size, sequence_length)`, *optional*):
                Segment token indices to indicate different portions of the inputs.
                Selected in the range ``[0, type_vocab_size - 1]``.
                If `type_vocab_size` is 2, which means the inputs have two portions.
                Indices can either be 0 or 1:
                - 0 corresponds to a *sentence A* token,
                - 1 corresponds to a *sentence B* token.
                Its data type should be `int64`. Defaults to `None`, which means we don't add segment embeddings.
            task_type_ids (`paddle.Tensor` of shape `(batch_size, sequence_length)`, *optional*):
                Indices of tasks of each input sequence tokens in the task embeddings (ErnieModel). Selected in
                the range ``[0, task_type_vocab_size - 1]``. Defaults to `None`.
            output_attentions (`bool`, *optional*):
                Whether or not to return the attentions tensors of all attention layers. See `attentions` under returned
                tensors for more detail.
            output_hidden_states (`bool`, *optional*):
                Whether or not to return the hidden states of all layers. See `hidden_states` under returned tensors for
                more detail.
            return_dict (`bool`, *optional*):
                Whether or not to return a [`BaseModelOutputWithPoolingAndCrossAttentions`] instead of a plain tuple.

        Returns:
            text_features (`paddle.Tensor` of shape `(batch_size, output_dim`): The text embeddings obtained by
            the pooled output of [`ErnieModel`].

        Example:
            .. code-block::

                from paddlenlp.transformers import ErnieViLModel, ErnieViLTokenizer

                model = ErnieViLModel.from_pretrained("PaddlePaddle/ernie_vil-2.0-base-zh")
                tokenizer = ErnieViLTokenizer.from_pretrained("PaddlePaddle/ernie_vil-2.0-base-zh")

                inputs = tokenizer(["一只猫的照片", "一条狗的照片"], padding=True, return_tensors="pd")
                text_features = model.get_text_features(**inputs)

        """
        output_attentions = output_attentions if output_attentions is not None else self.config.output_attentions
        output_hidden_states = (
            output_hidden_states if output_hidden_states is not None else self.config.output_hidden_states
        )
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        text_outputs = self.text_model(
            input_ids=input_ids,
            position_ids=position_ids,
            token_type_ids=token_type_ids,
            attention_mask=attention_mask,
            task_type_ids=task_type_ids,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict,
        )
        text_features = text_outputs[1]
        return text_features

    def forward(
        self,
        input_ids,
        pixel_values,
        attention_mask: Optional[paddle.Tensor] = None,
        position_ids: Optional[paddle.Tensor] = None,
        token_type_ids: Optional[paddle.Tensor] = None,
        task_type_ids: Optional[paddle.Tensor] = None,
        return_loss: Optional[bool] = None,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        return_dict: Optional[bool] = None,
    ) -> Union[Tuple, ErnieViLOutput]:
        r"""
        The ErnieViLModel forward method, overrides the `__call__()` special method.

        Args:
            input_ids (Tensor):
                Indices of input sequence tokens in the vocabulary. Padding will be ignored by default should you provide it.
                Its data type should be `int64` and it has a shape of [text_batch_size, sequence_length].
            pixel_values (Tensor):
                Pixel values. Padding will be ignored by default should you provide it.
                Its data type should be `float32` and it has a shape of [image_batch_size, num_channels, height, width].
            position_ids (Tensor, optional):
                Indices of positions of each input sequence tokens in the position embeddings (ErnieModel). Selected in
                the range ``[0, max_position_embeddings - 1]``.
                Shape as `(batch_size, num_tokens)` and dtype as int64. Defaults to `None`.
            token_type_ids (Tensor, optional):
                Segment token indices to indicate different portions of the inputs.
                Selected in the range ``[0, type_vocab_size - 1]``.
                If `type_vocab_size` is 2, which means the inputs have two portions.
                Indices can either be 0 or 1:

                - 0 corresponds to a *sentence A* token,
                - 1 corresponds to a *sentence B* token.

                Its data type should be `int64` and it has a shape of [batch_size, sequence_length].
                Defaults to `None`, which means we don't add segment embeddings.
            task_type_ids (Tensor, optional):
                Indices of tasks of each input sequence tokens in the task embeddings (ErnieModel). Selected in
                the range ``[0, task_type_vocab_size - 1]``.
                Shape as `(batch_size, sequence_length)` and dtype as int64. Defaults to `None`.
            attention_mask (Tensor, optional):
                Mask used in multi-head attention (ErnieModel) to avoid performing attention on to some unwanted positions,
                usually the paddings or the subsequent positions.
                Its data type can be int, float and bool.
                When the data type is bool, the `masked` tokens have `False` values and the others have `True` values.
                When the data type is int, the `masked` tokens have `0` values and the others have `1` values.
                When the data type is float, the `masked` tokens have `-INF` values and the others have `0` values.
                It is a tensor with shape broadcasted to `[batch_size, num_attention_heads, sequence_length, sequence_length]`.
                Defaults to `None`, which means nothing needed to be prevented attention to.
            output_hidden_states (bool, optional):
                Whether to return the hidden states of all layers.
                Defaults to `False`.
            output_attentions (bool, optional):
                Whether to return the attentions tensors of all attention layers.
                Defaults to `False`.
            return_dict (bool, optional):
                Whether to return a :class:`ErnieViLOutput` object. If `False`, the output
                will be a tuple of tensors. Defaults to `False`.

        Returns:
            An instance of :class:`ErnieViLOutput` if `return_dict=True`. Otherwise it returns a tuple of tensors
            corresponding to ordered and not None (depending on the input arguments) fields of :class:`ErnieViLOutput`.

        Example:
            .. code-block::

                import requests
                import paddle.nn.functional as F
                from PIL import Image
                from paddlenlp.transformers import ErnieViLModel, ErnieViLProcessor

                processor = ErnieViLProcessor.from_pretrained("PaddlePaddle/ernie_vil-2.0-base-zh")
                model = ErnieViLModel.from_pretrained("PaddlePaddle/ernie_vil-2.0-base-zh")
                model.eval()

                url = "http://images.cocodataset.org/val2017/000000039769.jpg"
                image = Image.open(requests.get(url, stream=True).raw)

                inputs = processor(text=["一只猫的照片", "一条狗的照片"],
                                images=image,
                                padding=True,
                                return_tensors="pd")

                outputs = model(**inputs)

                logits_per_image = outputs[0]
                probs = F.softmax(logits_per_image, axis=1)  # we can take the softmax to get the label probabilities

        """
        output_attentions = output_attentions if output_attentions is not None else self.config.output_attentions
        output_hidden_states = (
            output_hidden_states if output_hidden_states is not None else self.config.output_hidden_states
        )
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        vision_outputs = self.vision_model(
            pixel_values=pixel_values,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict,
        )

        text_outputs = self.text_model(
            input_ids=input_ids,
            attention_mask=attention_mask,
            token_type_ids=token_type_ids,
            task_type_ids=task_type_ids,
            position_ids=position_ids,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict,
        )
        image_embeds = vision_outputs[1]
        text_embeds = text_outputs[1]

        # normalized features
        image_embeds = F.normalize(image_embeds)
        text_embeds = F.normalize(text_embeds)

        if dist.get_world_size() > 1:
            feature_list_img = []
            feature_list_txt = []
            dist.all_gather(feature_list_img, image_embeds)
            dist.all_gather(feature_list_txt, text_embeds)
            image_embeds = paddle.concat(x=feature_list_img, axis=0)
            text_embeds = paddle.concat(x=feature_list_txt, axis=0)

        # cosine similarity as logits
        logit_scale = self.temperature.exp()

        logits_per_text = paddle.matmul(text_embeds * logit_scale, image_embeds, transpose_y=True)
        logits_per_image = logits_per_text.t()

        # clip temperature
        self.temperature.clip(-100.0, 100.0)

        loss = None

        if return_loss:
            loss = clip_loss(logits_per_text)

        if not return_dict:
            output = (logits_per_image, logits_per_text, text_embeds, image_embeds, text_outputs, vision_outputs)
            return ((loss,) + output) if loss is not None else output

        return ErnieViLOutput(
            loss=loss,
            logits_per_image=logits_per_image,
            logits_per_text=logits_per_text,
            text_embeds=text_embeds,
            image_embeds=image_embeds,
            text_model_output=text_outputs,
            vision_model_output=vision_outputs,
        )


class ErnieViLTextModel(ErnieViLPretrainedModel):
    r"""
    The text model from ErnieViL without any head or projection on top.

    This model inherits from :class:`~paddlenlp.transformers.model_utils.PretrainedModel`.
    Refer to the superclass documentation for the generic methods.
    This model is also a Paddle `paddle.nn.Layer <https://www.paddlepaddle.org.cn/documentation
    /docs/en/api/paddle/fluid/dygraph/layers/Layer_en.html>`__ subclass. Use it as a regular Paddle Layer
    and refer to the Paddle documentation for all matter related to general usage and behavior.

    Args:
        config (:class:`ErnieViLTextConfig`):
            An instance of ErnieViLTextConfig used to construct ErnieViLTextModel.
    """

    config_class = ErnieViLTextConfig

    def __init__(self, config: ErnieViLTextConfig):
        super().__init__(config)
        self.text_model = ErnieModel(config)

        self.init_weights()

    def get_input_embeddings(self) -> nn.Layer:
        return self.text_model.embeddings.word_embeddings

    def set_input_embeddings(self, value):
        self.text_model.embeddings.word_embeddings = value

    def forward(
        self,
        input_ids,
        attention_mask: Optional[paddle.Tensor] = None,
        position_ids: Optional[paddle.Tensor] = None,
        token_type_ids: Optional[paddle.Tensor] = None,
        task_type_ids: Optional[paddle.Tensor] = None,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        return_dict: Optional[bool] = None,
    ) -> Union[Tuple, BaseModelOutputWithPoolingAndCrossAttentions]:
        r"""
        Args:
            input_ids (`paddle.Tensor` of shape `(batch_size, sequence_length)`):
                Indices of input sequence tokens in the vocabulary. Padding will be ignored by default should you provide
                it.
                Indices can be obtained using [`ErnieViLTokenizer`].
            attention_mask (`paddle.Tensor` of shape `(batch_size, sequence_length)`, *optional*):
                Mask to avoid performing attention on padding token indices. Mask values selected in `[0, 1]`:
                - 1 for tokens that are **not masked**,
                - 0 for tokens that are **masked**.
            position_ids (`paddle.Tensor` of shape `(batch_size, sequence_length)`, *optional*):
                Indices of positions of each input sequence tokens in the position embeddings. Selected in the range `[0,
                config.max_position_embeddings - 1]`.
            token_type_ids (`paddle.Tensor` of shape `(batch_size, sequence_length)`, *optional*):
                Segment token indices to indicate different portions of the inputs.
                Selected in the range ``[0, type_vocab_size - 1]``.
                If `type_vocab_size` is 2, which means the inputs have two portions.
                Indices can either be 0 or 1:
                - 0 corresponds to a *sentence A* token,
                - 1 corresponds to a *sentence B* token.
                Its data type should be `int64`. Defaults to `None`, which means we don't add segment embeddings.
            task_type_ids (`paddle.Tensor` of shape `(batch_size, sequence_length)`, *optional*):
                Indices of tasks of each input sequence tokens in the task embeddings (ErnieModel). Selected in
                the range ``[0, task_type_vocab_size - 1]``. Defaults to `None`.
            output_attentions (`bool`, *optional*):
                Whether or not to return the attentions tensors of all attention layers. See `attentions` under returned
                tensors for more detail.
            output_hidden_states (`bool`, *optional*):
                Whether or not to return the hidden states of all layers. See `hidden_states` under returned tensors for
                more detail.
            return_dict (`bool`, *optional*):
                Whether or not to return a [`BaseModelOutputWithPoolingAndCrossAttentions`] instead of a plain tuple.

        Returns:
            An instance of :class:`BaseModelOutputWithPoolingAndCrossAttentions` if `return_dict=True`. Otherwise it returns a tuple of tensors
            corresponding to ordered and not None (depending on the input arguments) fields of :class:`BaseModelOutputWithPoolingAndCrossAttentions`.

        Examples:

        ```python
        >>> from paddlenlp.transformers import ErnieViLTokenizer, ErnieViLTextModel

        >>> model = ErnieViLTextModel.from_pretrained("PaddlePaddle/ernie_vil-2.0-base-zh")
        >>> tokenizer = ErnieViLTokenizer.from_pretrained("PaddlePaddle/ernie_vil-2.0-base-zh")

        >>> inputs = tokenizer(["一只猫的照片", "一条狗的照片"], padding=True, return_tensors="pd")

        >>> outputs = model(**inputs)
        >>> last_hidden_state = outputs.last_hidden_state
        >>> pooled_output = outputs.pooler_output  # pooled (EOS token) states
        ```
        """
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        return self.text_model(
            input_ids=input_ids,
            attention_mask=attention_mask,
            position_ids=position_ids,
            token_type_ids=token_type_ids,
            task_type_ids=task_type_ids,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict,
        )


class ErnieViLVisionModel(ErnieViLPretrainedModel):
    r"""
    The vision model from ErnieViL without any head or projection on top.

    This model inherits from :class:`~paddlenlp.transformers.model_utils.PretrainedModel`.
    Refer to the superclass documentation for the generic methods.
    This model is also a Paddle `paddle.nn.Layer <https://www.paddlepaddle.org.cn/documentation
    /docs/en/api/paddle/fluid/dygraph/layers/Layer_en.html>`__ subclass. Use it as a regular Paddle Layer
    and refer to the Paddle documentation for all matter related to general usage and behavior.

    Args:
        config (:class:`ErnieViLVisionConfig`):
            An instance of ErnieViLVisionConfig used to construct ErnieViLVisionModel.
    """

    config_class = ErnieViLVisionConfig
    main_input_name = "pixel_values"

    def __init__(self, config: ErnieViLVisionConfig):
        super().__init__(config)

        self.vision_model = ErnieViLVisionTransformer(config)

        self.init_weights()

    def get_input_embeddings(self) -> nn.Layer:
        return self.vision_model.conv1

    def forward(
        self,
        pixel_values: Optional[paddle.Tensor] = None,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        return_dict: Optional[bool] = None,
    ) -> Union[Tuple, BaseModelOutputWithPooling]:
        r"""
        Args:
            pixel_values (`paddle.Tensor` of shape `(batch_size, num_channels, height, width)`):
                Pixel values. Padding will be ignored by default should you provide it. Pixel values can be obtained using
                [`ErnieViLFeatureExtractor`]. See [`ErnieViLFeatureExtractor.__call__`] for details.
            output_attentions (`bool`, *optional*):
                Whether or not to return the attentions tensors of all attention layers. See `attentions` under returned
                tensors for more detail.
            output_hidden_states (`bool`, *optional*):
                Whether or not to return the hidden states of all layers. See `hidden_states` under returned tensors for
                more detail.
            return_dict (`bool`, *optional*):
                Whether or not to return a [`BaseModelOutputWithPooling`] instead of a plain tuple.

        Returns:
            An instance of :class:`BaseModelOutputWithPooling` if `return_dict=True`. Otherwise it returns a tuple of tensors
            corresponding to ordered and not None (depending on the input arguments) fields of :class:`BaseModelOutputWithPooling`.

        Examples:

        ```python
        >>> from PIL import Image
        >>> import requests
        >>> from paddlenlp.transformers import ErnieViLProcessor, ErnieViLVisionModel

        >>> model = ErnieViLVisionModel.from_pretrained("PaddlePaddle/ernie_vil-2.0-base-zh")
        >>> processor = ErnieViLProcessor.from_pretrained("PaddlePaddle/ernie_vil-2.0-base-zh")

        >>> url = "http://images.cocodataset.org/val2017/000000039769.jpg"
        >>> image = Image.open(requests.get(url, stream=True).raw)

        >>> inputs = processor(images=image, return_tensors="pd")

        >>> outputs = model(**inputs)
        >>> last_hidden_state = outputs.last_hidden_state
        >>> pooled_output = outputs.pooler_output  # pooled CLS states
        ```
        """
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        return self.vision_model(
            pixel_values=pixel_values,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict,
        )


class ErnieViLForImageGeneration(ErnieViLPretrainedModel, DiscoDiffusionMixin):
    r"""
    ErnieViL Model with diffusion model on top.

    This model inherits from :class:`~paddlenlp.transformers.model_utils.PretrainedModel`.
    Refer to the superclass documentation for the generic methods.
    This model is also a Paddle `paddle.nn.Layer <https://www.paddlepaddle.org.cn/documentation
    /docs/en/api/paddle/fluid/dygraph/layers/Layer_en.html>`__ subclass. Use it as a regular Paddle Layer
    and refer to the Paddle documentation for all matter related to general usage and behavior.

    Args:
        config (:class:`ErnieViLConfig`):
            An instance of ErnieViLConfig used to construct ErnieViLForImageGeneration.
    """

    config_class = ErnieViLConfig

    def __init__(self, config: ErnieViLConfig):
        super().__init__(config)
        self.ernie_vil = ErnieViLModel(config)
        self.unet_model = create_unet_model(
            image_size=512,
            num_channels=256,
            num_res_blocks=2,
            channel_mult="",
            learn_sigma=True,
            class_cond=False,
            attention_resolutions="32, 16, 8",
            num_heads=4,
            num_head_channels=64,
            num_heads_upsample=-1,
            use_scale_shift_norm=True,
            dropout=0.0,
            resblock_updown=True,
            use_new_attention_order=False,
        )
        self.secondary_model = create_secondary_model()

        # eval mode and stop all param's gradient
        self.eval()
        for param in self.parameters():
            param.stop_gradient = True

    def generate(
        self,
        input_ids,
        attention_mask=None,
        position_ids=None,
        token_type_ids=None,
        task_type_ids=None,
        init_image=None,
        output_dir="disco_diffusion_ernie_vil-2.0-base-zh/",
        width_height=[1280, 768],
        skip_steps=0,
        steps=250,
        cut_ic_pow=1,
        init_scale=1000,
        clip_guidance_scale=5000,
        tv_scale=0,
        range_scale=0,
        sat_scale=0,
        cutn_batches=4,
        perlin_init=False,
        perlin_mode="mixed",
        seed=None,
        eta=0.8,
        clamp_grad=True,
        clamp_max=0.05,
        cut_overview="[12]*400+[4]*600",
        cut_innercut="[4]*400+[12]*600",
        cut_icgray_p="[0.2]*400+[0]*600",
        save_rate=10,
        n_batches=1,
        batch_name="",
        use_secondary_model=True,
        randomize_class=True,
        clip_denoised=False,
    ):
        r"""
        The ErnieViLForImageGeneration generate method.

        Args:
            input_ids (Tensor):
                See :class:`ErnieViLModel`.
            attention_mask (Tensor, optional):
                See :class:`ErnieViLModel`.
            position_ids (Tensor, optional):
                See :class:`ErnieViLModel`.
            token_type_ids (Tensor, optional):
                See :class:`ErnieViLModel`.
            task_type_ids (Tensor, optional):
                See :class:`ErnieViLModel`.
            init_image (Path, optional):
                Recall that in the image sequence above, the first image shown is just noise.  If an init_image
                is provided, diffusion will replace the noise with the init_image as its starting state.  To use
                an init_image, upload the image to the Colab instance or your Google Drive, and enter the full
                image path here. If using an init_image, you may need to increase skip_steps to ~ 50% of total
                steps to retain the character of the init. See skip_steps above for further discussion.
                Default to `None`.
            output_dir (Path, optional):
                Output directory.
                Default to `disco_diffusion_ernie_vil-2.0-base-zh/`.
            width_height (List[int, int], optional):
                Desired final image size, in pixels. You can have a square, wide, or tall image, but each edge
                length should be set to a multiple of 64px, and a minimum of 512px on the default ErnieViL model setting.
                If you forget to use multiples of 64px in your dimensions, DD will adjust the dimensions of your
                image to make it so.
                Default to `[1280, 768]`.
            skip_steps (int, optional):
                Consider the chart shown here.  Noise scheduling (denoise strength) starts very high and progressively
                gets lower and lower as diffusion steps progress. The noise levels in the first few steps are very high,
                so images change dramatically in early steps.As DD moves along the curve, noise levels (and thus the
                amount an image changes per step) declines, and image coherence from one step to the next increases.
                The first few steps of denoising are often so dramatic that some steps (maybe 10-15% of total) can be
                skipped without affecting the final image. You can experiment with this as a way to cut render times.
                If you skip too many steps, however, the remaining noise may not be high enough to generate new content,
                and thus may not have time left to finish an image satisfactorily.Also, depending on your other settings,
                you may need to skip steps to prevent ErnieViL from overshooting your goal, resulting in blown out colors
                (hyper saturated, solid white, or solid black regions) or otherwise poor image quality.  Consider that
                the denoising process is at its strongest in the early steps, so skipping steps can sometimes mitigate
                other problems.Lastly, if using an init_image, you will need to skip ~50% of the diffusion steps to retain
                the shapes in the original init image. However, if you're using an init_image, you can also adjust
                skip_steps up or down for creative reasons.  With low skip_steps you can get a result "inspired by"
                the init_image which will retain the colors and rough layout and shapes but look quite different.
                With high skip_steps you can preserve most of the init_image contents and just do fine tuning of the texture.
                Default to `0`.
            steps:
                When creating an image, the denoising curve is subdivided into steps for processing. Each step (or iteration)
                involves the AI looking at subsets of the image called 'cuts' and calculating the 'direction' the image
                should be guided to be more like the prompt. Then it adjusts the image with the help of the diffusion denoiser,
                and moves to the next step.Increasing steps will provide more opportunities for the AI to adjust the image,
                and each adjustment will be smaller, and thus will yield a more precise, detailed image.  Increasing steps
                comes at the expense of longer render times.  Also, while increasing steps should generally increase image
                quality, there is a diminishing return on additional steps beyond 250 - 500 steps.  However, some intricate
                images can take 1000, 2000, or more steps.  It is really up to the user.  Just know that the render time is
                directly related to the number of steps, and many other parameters have a major impact on image quality, without
                costing additional time.
            cut_ic_pow (int, optional):
                This sets the size of the border used for inner cuts.  High cut_ic_pow values have larger borders, and
                therefore the cuts themselves will be smaller and provide finer details.  If you have too many or too-small
                inner cuts, you may lose overall image coherency and/or it may cause an undesirable 'mosaic' effect.
                Low cut_ic_pow values will allow the inner cuts to be larger, helping image coherency while still helping
                with some details.
                Default to `1`.
            init_scale (int, optional):
                This controls how strongly ErnieViL will try to match the init_image provided.  This is balanced against the
                clip_guidance_scale (CGS) above.  Too much init scale, and the image won't change much during diffusion.
                Too much CGS and the init image will be lost.
                Default to `1000`.
            clip_guidance_scale (int, optional):
                CGS is one of the most important parameters you will use. It tells DD how strongly you want ErnieViL to move
                toward your prompt each timestep.  Higher is generally better, but if CGS is too strong it will overshoot
                the goal and distort the image. So a happy medium is needed, and it takes experience to learn how to adjust
                CGS. Note that this parameter generally scales with image dimensions. In other words, if you increase your
                total dimensions by 50% (e.g. a change from 512 x 512 to 512 x 768), then to maintain the same effect on the
                image, you'd want to increase clip_guidance_scale from 5000 to 7500. Of the basic settings, clip_guidance_scale,
                steps and skip_steps are the most important contributors to image quality, so learn them well.
                Default to `5000`.
            tv_scale (int, optional):
                Total variance denoising. Optional, set to zero to turn off. Controls smoothness of final output. If used,
                tv_scale will try to smooth out your final image to reduce overall noise. If your image is too 'crunchy',
                increase tv_scale. TV denoising is good at preserving edges while smoothing away noise in flat regions.
                See https://en.wikipedia.org/wiki/Total_variation_denoising
                Default to `0`.
            range_scale (int, optional):
                Optional, set to zero to turn off.  Used for adjustment of color contrast.  Lower range_scale will increase
                contrast. Very low numbers create a reduced color palette, resulting in more vibrant or poster-like images.
                Higher range_scale will reduce contrast, for more muted images.
                Default to `0`.
            sat_scale (int, optional):
                Saturation scale. Optional, set to zero to turn off.  If used, sat_scale will help mitigate oversaturation.
                If your image is too saturated, increase sat_scale to reduce the saturation.
                Default to `0`.
            cutn_batches (int, optional):
                Each iteration, the AI cuts the image into smaller pieces known as cuts, and compares each cut to the prompt
                to decide how to guide the next diffusion step.  More cuts can generally lead to better images, since DD has
                more chances to fine-tune the image precision in each timestep.  Additional cuts are memory intensive, however,
                and if DD tries to evaluate too many cuts at once, it can run out of memory.  You can use cutn_batches to increase
                cuts per timestep without increasing memory usage. At the default settings, DD is scheduled to do 16 cuts per
                timestep.  If cutn_batches is set to 1, there will indeed only be 16 cuts total per timestep. However, if
                cutn_batches is increased to 4, DD will do 64 cuts total in each timestep, divided into 4 sequential batches
                of 16 cuts each.  Because the cuts are being evaluated only 16 at a time, DD uses the memory required for only 16 cuts,
                but gives you the quality benefit of 64 cuts.  The tradeoff, of course, is that this will take ~4 times as long to
                render each image.So, (scheduled cuts) x (cutn_batches) = (total cuts per timestep). Increasing cutn_batches will
                increase render times, however, as the work is being done sequentially.  DD's default cut schedule is a good place
                to start, but the cut schedule can be adjusted in the Cutn Scheduling section, explained below.
                Default to `4`.
            perlin_init (bool, optional):
                Normally, DD will use an image filled with random noise as a starting point for the diffusion curve.
                If perlin_init is selected, DD will instead use a Perlin noise model as an initial state.  Perlin has very
                interesting characteristics, distinct from random noise, so it's worth experimenting with this for your projects.
                Beyond perlin, you can, of course, generate your own noise images (such as with GIMP, etc) and use them as an
                init_image (without skipping steps). Choosing perlin_init does not affect the actual diffusion process, just the
                starting point for the diffusion. Please note that selecting a perlin_init will replace and override any init_image
                you may have specified. Further, because the 2D, 3D and video animation systems all rely on the init_image system,
                if you enable Perlin while using animation modes, the perlin_init will jump in front of any previous image or video
                input, and DD will NOT give you the expected sequence of coherent images. All of that said, using Perlin and
                animation modes together do make a very colorful rainbow effect, which can be used creatively.
                Default to `False`.
            perlin_mode (str, optional):
                sets type of Perlin noise: colored, gray, or a mix of both, giving you additional options for noise types. Experiment
                to see what these do in your projects.
                Default to `mixed`.
            seed (int, optional):
                Deep in the diffusion code, there is a random number seed which is used as the basis for determining the initial
                state of the diffusion.  By default, this is random, but you can also specify your own seed. This is useful if you like a
                particular result and would like to run more iterations that will be similar. After each run, the actual seed value used will be
                reported in the parameters report, and can be reused if desired by entering seed # here.  If a specific numerical seed is used
                repeatedly, the resulting images will be quite similar but not identical.
                Default to `None`.
            eta (float, optional):
                Eta (greek letter η) is a diffusion model variable that mixes in a random amount of scaled noise into each timestep.
                0 is no noise, 1.0 is more noise. As with most DD parameters, you can go below zero for eta, but it may give you
                unpredictable results. The steps parameter has a close relationship with the eta parameter. If you set eta to 0,
                then you can get decent output with only 50-75 steps. Setting eta to 1.0 favors higher step counts, ideally around
                250 and up. eta has a subtle, unpredictable effect on image, so you'll need to experiment to see how this affects your projects.
                Default to `0.8`.
            clamp_grad (bool, optional):
                As I understand it, clamp_grad is an internal limiter that stops DD from producing extreme results. Try your images with and without
                clamp_grad. If the image changes drastically with clamp_grad turned off, it probably means your clip_guidance_scale is too high and
                should be reduced.
                Default to `True`.
            clamp_max (float, optional):
                Sets the value of the clamp_grad limitation. Default is 0.05, providing for smoother, more muted coloration in images, but setting
                higher values (0.15-0.3) can provide interesting contrast and vibrancy.
                Default to `0.05`.
            cut_overview (str, optional):
                The schedule of overview cuts.
                Default to `'[12]*400+[4]*600'`.
            cut_innercut (str, optional):
                The schedule of inner cuts.
                Default to `'[4]*400+[12]*600'`.
            cut_icgray_p (str, optional):
                This sets the size of the border used for inner cuts.  High cut_ic_pow values have larger borders, and therefore the cuts
                themselves will be smaller and provide finer details.  If you have too many or too-small inner cuts, you may lose overall
                image coherency and/or it may cause an undesirable 'mosaic' effect.   Low cut_ic_pow values will allow the inner cuts to be
                larger, helping image coherency while still helping with some details.
                Default to `'[0.2]*400+[0]*600'`.
            save_rate (int, optional):
                During a diffusion run, you can monitor the progress of each image being created with this variable.  If display_rate is set
                to 50, DD will show you the in-progress image every 50 timesteps. Setting this to a lower value, like 5 or 10, is a good way
                to get an early peek at where your image is heading. If you don't like the progression, just interrupt execution, change some
                settings, and re-run.  If you are planning a long, unmonitored batch, it's better to set display_rate equal to steps, because
                displaying interim images does slow Colab down slightly.
                Default to `10`.
            n_batches (int, optional):
                This variable sets the number of still images you want DD to create.  If you are using an animation mode (see below for details)
                DD will ignore n_batches and create a single set of animated frames based on the animation settings.
                Default to `1`.
            batch_name (str, optional):
                The name of the batch, the batch id will be named as "progress-[batch_name]-seed-[range(n_batches)]-[save_rate]". To avoid your
                artworks be overridden by other users, please use a unique name.
                Default to `''`.
            use_secondary_model (bool, optional):
                Whether or not use secondary model.
                Default to `True`.
            randomize_class (bool, optional):
                Random class.
                Default to `True`.
            clip_denoised (bool, optional):
                Clip denoised.
                Default to `False`.

        Returns:
            List[PIL.Image]: Returns n_batches of final image.
            Its data type should be PIL.Image.

        Example:
            .. code-block::

            from paddlenlp.transformers import ErnieViLForImageGeneration, ErnieViLTokenizer

            # Initialize the model and tokenizer
            model_name_or_path = "PaddlePaddle/disco_diffusion_ernie_vil-2.0-base-zh"
            model = ErnieViLForImageGeneration.from_pretrained(model_name_or_path)
            tokenizer = ErnieViLTokenizer.from_pretrained(model_name_or_path)
            model.eval()

            # Prepare the model inputs.
            text_prompt = "小桥流水人家。"
            style = None
            artist = None
            text_prompt = model.preprocess_text_prompt(text_prompt, style=style, artist=artist)
            tokenized_inputs = tokenizer(text_prompt, return_tensors="pd", padding="max_length", max_length=tokenizer.model_max_length)

            images = model.generate(**tokenized_inputs)
            # return List[PIL.Image]
            images[0].save("figure.png")

        """
        self.diffusion = create_gaussian_diffusion(
            steps=steps,
            learn_sigma=True,
            sigma_small=False,
            noise_schedule="linear",
            predict_xstart=False,
            rescale_timesteps=True,
        )
        target_text_embeds = self.ernie_vil.get_text_features(
            input_ids=input_ids,
            attention_mask=attention_mask,
            position_ids=position_ids,
            token_type_ids=token_type_ids,
            task_type_ids=task_type_ids,
        )

        images_list = super().disco_diffusion_generate(
            target_text_embeds=target_text_embeds,
            clamp_grad=clamp_grad,
            clamp_max=clamp_max,
            clip_denoised=clip_denoised,
            clip_guidance_scale=clip_guidance_scale,
            cut_ic_pow=cut_ic_pow,
            cut_icgray_p=cut_icgray_p,
            cut_innercut=cut_innercut,
            cut_overview=cut_overview,
            cutn_batches=cutn_batches,
            save_rate=save_rate,
            eta=eta,
            init_image=init_image,
            init_scale=init_scale,
            n_batches=n_batches,
            output_dir=output_dir,
            perlin_init=perlin_init,
            perlin_mode=perlin_mode,
            randomize_class=randomize_class,
            range_scale=range_scale,
            sat_scale=sat_scale,
            seed=seed,
            skip_steps=skip_steps,
            tv_scale=tv_scale,
            use_secondary_model=use_secondary_model,
            width_height=width_height,
            image_mean=[0.485, 0.456, 0.406],
            image_std=[0.229, 0.224, 0.225],
            batch_name=batch_name,
        )

        return images_list

    def preprocess_text_prompt(self, text_prompt, style=None, artist=None):
        text_prompt = text_prompt.rstrip(",.，。")
        if style is not None:
            text_prompt += "，{}".format(style)
        if artist is not None:
            text_prompt += "，由{}所作".format(artist)
        return text_prompt



# Copyright (c) 2020 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import numpy as np

import paddle
from .utils import default_trans_func

__all__ = ["RougeL", "RougeLForDuReader"]


class RougeN:
    def __init__(self, n):
        self.n = n

    def _get_ngrams(self, words):
        """Calculates word n-grams for multiple sentences."""
        ngram_set = set()
        max_index_ngram_start = len(words) - self.n
        for i in range(max_index_ngram_start + 1):
            ngram_set.add(tuple(words[i : i + self.n]))
        return ngram_set

    def score(self, evaluated_sentences_ids, reference_sentences_ids):
        overlapping_count, reference_count = self.compute(evaluated_sentences_ids, reference_sentences_ids)
        return overlapping_count / reference_count

    def compute(self, evaluated_sentences_ids, reference_sentences_ids):
        """
        Args:
            evaluated_sentences (list): the sentences ids predicted by the model.
            reference_sentences (list): the referenced sentences ids. Its size should be same as evaluated_sentences.

        Returns:
            overlapping_count (int): the overlapping n-gram count.
            reference_count (int): the reference sentences n-gram count.
        """
        if len(evaluated_sentences_ids) <= 0 or len(reference_sentences_ids) <= 0:
            raise ValueError("Collections must contain at least 1 sentence.")

        reference_count = 0
        overlapping_count = 0

        for evaluated_sentence_ids, reference_sentence_ids in zip(evaluated_sentences_ids, reference_sentences_ids):
            evaluated_ngrams = self._get_ngrams(evaluated_sentence_ids)
            reference_ngrams = self._get_ngrams(reference_sentence_ids)
            reference_count += len(reference_ngrams)

            # Gets the overlapping ngrams between evaluated and reference
            overlapping_ngrams = evaluated_ngrams.intersection(reference_ngrams)
            overlapping_count += len(overlapping_ngrams)

        return overlapping_count, reference_count

    def accumulate(self):
        """
        This function returns the mean precision, recall and f1 score for all accumulated minibatches.

        Returns:
            float: mean precision, recall and f1 score.
        """
        rouge_score = self.overlapping_count / self.reference_count
        return rouge_score

    def reset(self):
        """
        Reset function empties the evaluation memory for previous mini-batches.
        """
        self.overlapping_count = 0
        self.reference_count = 0

    def name(self):
        """
        Return name of metric instance.
        """
        return "Rouge-%s" % self.n

    def update(self, overlapping_count, reference_count):
        """
        Args:
        """
        self.overlapping_count += overlapping_count
        self.reference_count += reference_count


class Rouge1(RougeN):
    def __init__(self):
        super(Rouge1, self).__init__(n=1)


class Rouge2(RougeN):
    def __init__(self):
        super(Rouge2, self).__init__(n=2)


class RougeL(paddle.metric.Metric):
    r"""
    Rouge-L is Recall-Oriented Understudy for Gisting Evaluation based on Longest Common Subsequence (LCS).
    Longest common subsequence problem takes into account sentence level structure
    similarity naturally and identifies longest co-occurring
    in sequence n-grams automatically.

    .. math::

        R_{LCS} & = \frac{LCS(C,S)}{len(S)}

        P_{LCS} & = \frac{LCS(C,S)}{len(C)}

        F_{LCS} & = \frac{(1 + \gamma^2)R_{LCS}P_{LCS}}{R_{LCS}} + \gamma^2{R_{LCS}}

    where `C` is the candidate sentence, and `S` is the reference sentence.

    Args:
        trans_func (callable, optional): `trans_func` transforms the network
            output to string to calculate.
        vocab (dict|paddlenlp.data.vocab, optional): Vocab for target language.
            If `trans_func` is None and RougeL is used as `paddle.metric.Metric`
            instance, `default_trans_func` will be performed and `vocab` must
            be provided.
        gamma (float): A hyperparameter to decide the weight of recall. Defaults to 1.2.
        name (str, optional): Name of `paddle.metric.Metric` instance. Defaults to "rouge-l".

    Examples:
        .. code-block:: python

            from paddlenlp.metrics import RougeL
            rougel = RougeL()
            cand = ["The","cat","The","cat","on","the","mat"]
            ref_list = [["The","cat","is","on","the","mat"], ["There","is","a","cat","on","the","mat"]]
            rougel.add_inst(cand, ref_list)
            print(rougel.score()) # 0.7800511508951408

    """

    def __init__(self, trans_func=None, vocab=None, gamma=1.2, name="rouge-l", *args, **kwargs):
        super(RougeL, self).__init__(*args, **kwargs)
        self.gamma = gamma
        self.inst_scores = []
        self._name = name
        self.vocab = vocab
        self.trans_func = trans_func

    def lcs(self, string, sub):
        """
        Calculate the length of longest common subsequence of string and sub.

        Args:
            string (str):
                The string to be calculated, usually longer the sub string.
            sub (str):
                The sub string to be calculated.

        Returns:
            float: Returns the length of the longest common subsequence of string and sub.
        """
        if len(string) < len(sub):
            sub, string = string, sub
        lengths = np.zeros((len(string) + 1, len(sub) + 1))
        for j in range(1, len(sub) + 1):
            for i in range(1, len(string) + 1):
                if string[i - 1] == sub[j - 1]:
                    lengths[i][j] = lengths[i - 1][j - 1] + 1
                else:
                    lengths[i][j] = max(lengths[i - 1][j], lengths[i][j - 1])
        return lengths[len(string)][len(sub)]

    def add_inst(self, cand, ref_list):
        """
        Update the states based on the a pair of candidate and references.

        Args:
            cand (str): The candidate sentence generated by model.
            ref_list (list): List of ground truth sentences.
        """
        precs, recalls = [], []
        for ref in ref_list:
            basic_lcs = self.lcs(cand, ref)
            prec = basic_lcs / len(cand) if len(cand) > 0.0 else 0.0
            rec = basic_lcs / len(ref) if len(ref) > 0.0 else 0.0
            precs.append(prec)
            recalls.append(rec)

        prec_max = max(precs)
        rec_max = max(recalls)

        if prec_max != 0 and rec_max != 0:
            score = ((1 + self.gamma**2) * prec_max * rec_max) / float(rec_max + self.gamma**2 * prec_max)
        else:
            score = 0.0
        self.inst_scores.append(score)

    def update(self, output, label, seq_mask=None):
        if self.trans_func is None:
            if self.vocab is None:
                raise AttributeError(
                    "The `update` method requires users to provide `trans_func` or `vocab` when initializing RougeL."
                )
            cand_list, ref_list = default_trans_func(output, label, seq_mask, self.vocab)
        else:
            cand_list, ref_list = self.trans_func(output, label, seq_mask)
        if len(cand_list) != len(ref_list):
            raise ValueError("Length error! Please check the output of network.")
        for i in range(len(cand_list)):
            self.add_inst(cand_list[i], ref_list[i])

    def accumulate(self):
        """
        Calculate the final rouge-l metric.
        """
        return 1.0 * sum(self.inst_scores) / len(self.inst_scores)

    def score(self):
        return self.accumulate()

    def reset(self):
        self.inst_scores = []

    def name(self):
        return self._name


class RougeLForDuReader(RougeL):
    """
    Rouge-L metric with bonus for DuReader contest.

    Please refer to `DuReader Homepage<https://ai.baidu.com//broad/subordinate?dataset=dureader>`_ for more details.

    Args:
        alpha (float, optional): Weight of YesNo dataset when adding bonus for DuReader contest. Defaults to 1.0.
        beta (float, optional): Weight of Entity dataset when adding bonus for DuReader contest. Defaults to 1.0.
    """

    def __init__(self, alpha=1.0, beta=1.0, gamma=1.2):
        super(RougeLForDuReader, self).__init__(gamma)
        self.alpha = alpha
        self.beta = beta

    def add_inst(self, cand, ref_list, yn_label=None, yn_ref=None, entity_ref=None):
        precs, recalls = [], []
        for i, ref in enumerate(ref_list):
            basic_lcs = self.lcs(cand, ref)
            yn_bonus, entity_bonus = 0.0, 0.0
            if yn_ref is not None and yn_label is not None:
                yn_bonus = self.add_yn_bonus(cand, ref, yn_label, yn_ref[i])
            elif entity_ref is not None:
                entity_bonus = self.add_entity_bonus(cand, entity_ref)
            p_denom = len(cand) + self.alpha * yn_bonus + self.beta * entity_bonus
            r_denom = len(ref) + self.alpha * yn_bonus + self.beta * entity_bonus
            prec = (basic_lcs + self.alpha * yn_bonus + self.beta * entity_bonus) / p_denom if p_denom > 0.0 else 0.0
            rec = (basic_lcs + self.alpha * yn_bonus + self.beta * entity_bonus) / r_denom if r_denom > 0.0 else 0.0
            precs.append(prec)
            recalls.append(rec)

        prec_max = max(precs)
        rec_max = max(recalls)
        if prec_max != 0 and rec_max != 0:
            score = ((1 + self.gamma**2) * prec_max * rec_max) / float(rec_max + self.gamma**2 * prec_max)
        else:
            score = 0.0
        self.inst_scores.append(score)

    def add_yn_bonus(self, cand, ref, yn_label, yn_ref):
        if yn_label != yn_ref:
            return 0.0
        lcs_ = self.lcs(cand, ref)
        return lcs_

    def add_entity_bonus(self, cand, entity_ref):
        lcs_ = 0.0
        for ent in entity_ref:
            if ent in cand:
                lcs_ += len(ent)
        return lcs_



# Copyright (c) 2022 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
import os
import warnings
import json

from paddle.io import Dataset
from paddle.dataset.common import md5file
from paddle.utils.download import get_path_from_url
from paddlenlp.utils.env import DATA_HOME
from paddlenlp.datasets import DatasetBuilder

__all__ = ["CAIL2018Small"]


class CAIL2018Small(DatasetBuilder):
    """
    CAIL2018-Small 196,000 criminal cases，which are collected from http://wenshu.court.gov.cn/
    published by the Supreme People’s Court of China. Each case in CAIL2018 consists of two parts,
    i.e., fact description and corresponding judgment result. The judgment result of each case is
    refined into 3 representative ones, including relevant law articles, charges, and prison terms.

    charges: predict the charges from referee result with regular expressions.

    law_articles: predict the relevant law articles from referee result with regular expressions.

    prison_term: predict the prison terms from referee result with regular expressions.

    Find more dataset dertails in https://github.com/thunlp/CAIL
    """

    lazy = False
    URL = "https://paddlenlp.bj.bcebos.com/datasets/cail2018_small.tar.gz"
    MD5 = "963401d107150e250580d115dd2d43fc"
    META_INFO = collections.namedtuple("META_INFO", ("file", "md5"))
    SPLITS = {
        "train": META_INFO(os.path.join("cail2018_small", "train.json"), "e11fc099cc7709a8d128e9fe9f029621"),
        "dev": META_INFO(os.path.join("cail2018_small", "dev.json"), "ee13108aee6a08a94490fadeb400debb"),
        "test": META_INFO(os.path.join("cail2018_small", "test.json"), "27cea977fff2f85b5c32a8e0f708b093"),
    }

    def _get_data(self, mode, **kwargs):
        """Check and download Dataset"""
        default_root = os.path.join(DATA_HOME, self.__class__.__name__)
        filename, data_hash = self.SPLITS[mode]
        fullname = os.path.join(default_root, filename)
        if not os.path.exists(fullname) or (data_hash and not md5file(fullname) == data_hash):

            get_path_from_url(self.URL, default_root, self.MD5)

        return fullname

    def _read(self, filename, *args):

        with open(filename, "r", encoding="utf-8") as f:
            for line in f.readlines():
                line = json.loads(line)
                sentence = line["fact"]
                if self.name == "charges":
                    label = line["meta"]["accusation"]
                    yield {"sentence": sentence, "label": label}
                elif self.name == "law_articles":
                    label = line["meta"]["relevant_articles"]
                    yield {"sentence": sentence, "label": label}
                elif self.name == "prison_term":
                    if line["meta"]["term_of_imprisonment"]["life_imprisonment"]:
                        lp = -1
                    elif line["meta"]["term_of_imprisonment"]["death_penalty"]:
                        lp = -2
                    else:
                        lp = line["meta"]["term_of_imprisonment"]["imprisonment"]
                    yield {"sentence": sentence, "label": lp}
                else:
                    assert "Dataset name {} does not exist".format(self.name)
        f.close()

    def get_labels(self):
        """
        Return labels of the CAIL2018-Small.
        """
        if self.name == "charges":
            return [
                "故意伤害",
                "盗窃",
                "危险驾驶",
                "非法[持有、私藏][枪支、弹药]",
                "交通肇事",
                "寻衅滋事",
                "[窝藏、包庇]",
                "放火",
                "故意毁坏财物",
                "绑架",
                "赌博",
                "妨害公务",
                "合同诈骗",
                "[走私、贩卖、运输、制造]毒品",
                "抢劫",
                "非法拘禁",
                "诬告陷害",
                "非法采矿",
                "容留他人吸毒",
                "强奸",
                "[伪造、变造、买卖]国家机关[公文、证件、印章]",
                "故意杀人",
                "诈骗",
                "聚众斗殴",
                "[掩饰、隐瞒][犯罪所得、犯罪所得收益]",
                "敲诈勒索",
                "[组织、强迫、引诱、容留、介绍]卖淫",
                "[引诱、容留、介绍]卖淫",
                "开设赌场",
                "重大责任事故",
                "抢夺",
                "破坏电力设备",
                "[制造、贩卖、传播]淫秽物品",
                "传播淫秽物品",
                "虐待",
                "非法[采伐、毁坏]国家重点保护植物",
                "非法[制造、买卖、运输、邮寄、储存][枪支、弹药、爆炸物]",
                "受贿",
                "脱逃",
                "行贿",
                "破坏[广播电视设施、公用电信设施]",
                "[伪造、变造]居民身份证",
                "拐卖[妇女、儿童]",
                "强迫交易",
                "拒不支付劳动报酬",
                "帮助[毁灭、伪造]证据",
                "爆炸",
                "污染环境",
                "非法持有毒品",
                "破坏易燃易爆设备",
                "妨害信用卡管理",
                "[引诱、教唆、欺骗]他人吸毒",
                "非法处置[查封、扣押、冻结]的财产",
                "贪污",
                "职务侵占",
                "帮助犯罪分子逃避处罚",
                "盗伐林木",
                "挪用资金",
                "重婚",
                "侵占",
                "[窝藏、转移、收购、销售]赃物",
                "妨害作证",
                "挪用公款",
                "伪造[公司、企业、事业单位、人民团体]印章",
                "[窝藏、转移、隐瞒][毒品、毒赃]",
                "[虚开增值税专用发票、用于骗取出口退税、抵扣税款发票]",
                "非法侵入住宅",
                "信用卡诈骗",
                "非法获取公民个人信息",
                "滥伐林木",
                "非法经营",
                "招摇撞骗",
                "以危险方法危害公共安全",
                "[盗窃、侮辱]尸体",
                "过失致人死亡",
                "[持有、使用]假币",
                "传授犯罪方法",
                "猥亵儿童",
                "逃税",
                "非法吸收公众存款",
                "非法[转让、倒卖]土地使用权",
                "骗取[贷款、票据承兑、金融票证]",
                "破坏生产经营",
                "高利转贷",
                "[盗窃、抢夺][枪支、弹药、爆炸物]",
                "[盗窃、抢夺][枪支、弹药、爆炸物、危险物质]",
                "假冒注册商标",
                "[伪造、变造]金融票证",
                "强迫卖淫",
                "扰乱无线电通讯管理秩序",
                "虚开发票",
                "非法占用农用地",
                "[组织、领导、参加]黑社会性质组织",
                "[隐匿、故意销毁][会计凭证、会计帐簿、财务会计报告]",
                "保险诈骗",
                "强制[猥亵、侮辱]妇女",
                "非国家工作人员受贿",
                "伪造货币",
                "拒不执行[判决、裁定]",
                "[生产、销售]伪劣产品",
                "非法[收购、运输][盗伐、滥伐]的林木",
                "冒充军人招摇撞骗",
                "组织卖淫",
                "持有伪造的发票",
                "[生产、销售][有毒、有害]食品",
                "非法[制造、出售]非法制造的发票",
                "[伪造、变造、买卖]武装部队[公文、证件、印章]",
                "[组织、领导]传销活动",
                "强迫劳动",
                "走私",
                "贷款诈骗",
                "串通投标",
                "虚报注册资本",
                "侮辱",
                "伪证",
                "聚众扰乱社会秩序",
                "聚众扰乱[公共场所秩序、交通秩序]",
                "劫持[船只、汽车]",
                "集资诈骗",
                "盗掘[古文化遗址、古墓葬]",
                "失火",
                "票据诈骗",
                "经济犯",
                "单位行贿",
                "投放危险物质",
                "过失致人重伤",
                "破坏交通设施",
                "聚众哄抢",
                "走私普通[货物、物品]",
                "收买被拐卖的[妇女、儿童]",
                "非法狩猎",
                "销售假冒注册商标的商品",
                "破坏监管秩序",
                "拐骗儿童",
                "非法行医",
                "协助组织卖淫",
                "打击报复证人",
                "强迫他人吸毒",
                "非法[收购、运输、加工、出售][国家重点保护植物、国家重点保护植物制品]",
                "[生产、销售]不符合安全标准的食品",
                "非法买卖制毒物品",
                "滥用职权",
                "聚众冲击国家机关",
                "[出售、购买、运输]假币",
                "对非国家工作人员行贿",
                "[编造、故意传播]虚假恐怖信息",
                "玩忽职守",
                "私分国有资产",
                "非法携带[枪支、弹药、管制刀具、危险物品]危及公共安全",
                "过失以危险方法危害公共安全",
                "走私国家禁止进出口的[货物、物品]",
                "违法发放贷款",
                "徇私枉法",
                "非法[买卖、运输、携带、持有]毒品原植物[种子、幼苗]",
                "动植物检疫徇私舞弊",
                "重大劳动安全事故",
                "走私[武器、弹药]",
                "破坏计算机信息系统",
                "[制作、复制、出版、贩卖、传播]淫秽物品牟利",
                "单位受贿",
                "[生产、销售]伪劣[农药、兽药、化肥、种子]",
                "过失损坏[武器装备、军事设施、军事通信]",
                "破坏交通工具",
                "包庇毒品犯罪分子",
                "[生产、销售]假药",
                "非法种植毒品原植物",
                "诽谤",
                "传播性病",
                "介绍贿赂",
                "金融凭证诈骗",
                "非法[猎捕、杀害][珍贵、濒危]野生动物",
                "徇私舞弊不移交刑事案件",
                "巨额财产来源不明",
                "过失损坏[广播电视设施、公用电信设施]",
                "挪用特定款物",
                "[窃取、收买、非法提供]信用卡信息",
                "非法组织卖血",
                "利用影响力受贿",
                "非法捕捞水产品",
                "对单位行贿",
                "遗弃",
                "徇私舞弊[不征、少征]税款",
                "提供[侵入、非法控制计算机信息系统][程序、工具]",
                "非法进行节育手术",
                "危险物品肇事",
                "非法[制造、买卖、运输、储存]危险物质",
                "非法[制造、销售]非法制造的注册商标标识",
                "侵犯著作权",
                "倒卖[车票、船票]",
                "过失投放危险物质",
                "走私废物",
                "非法出售发票",
                "走私[珍贵动物、珍贵动物制品]",
                "[伪造、倒卖]伪造的有价票证",
                "招收[公务员、学生]徇私舞弊",
                "非法[生产、销售]间谍专用器材",
                "倒卖文物",
                "虐待被监管人",
                "洗钱",
                "非法[生产、买卖]警用装备",
                "非法获取国家秘密",
                "非法[收购、运输、出售][珍贵、濒危野生动物、珍贵、濒危野生动物]制品",
            ]
        elif self.name == "law_articles":
            return [
                114,
                115,
                116,
                117,
                118,
                119,
                122,
                124,
                125,
                127,
                128,
                130,
                132,
                133,
                134,
                135,
                136,
                140,
                141,
                143,
                144,
                147,
                149,
                150,
                151,
                152,
                153,
                155,
                156,
                158,
                159,
                161,
                162,
                163,
                164,
                168,
                170,
                171,
                172,
                175,
                176,
                177,
                184,
                185,
                186,
                191,
                192,
                193,
                194,
                196,
                198,
                199,
                200,
                201,
                205,
                209,
                210,
                211,
                212,
                213,
                214,
                215,
                217,
                220,
                223,
                224,
                225,
                226,
                227,
                228,
                231,
                232,
                233,
                234,
                235,
                236,
                237,
                238,
                239,
                240,
                241,
                243,
                244,
                245,
                246,
                248,
                253,
                258,
                260,
                261,
                262,
                263,
                264,
                266,
                267,
                268,
                269,
                270,
                271,
                272,
                273,
                274,
                275,
                276,
                277,
                279,
                280,
                281,
                282,
                283,
                285,
                286,
                288,
                290,
                291,
                292,
                293,
                294,
                295,
                302,
                303,
                305,
                307,
                308,
                310,
                312,
                313,
                314,
                315,
                316,
                326,
                328,
                333,
                336,
                338,
                340,
                341,
                342,
                343,
                344,
                345,
                346,
                347,
                348,
                349,
                350,
                351,
                352,
                353,
                354,
                356,
                357,
                358,
                359,
                360,
                361,
                363,
                364,
                367,
                369,
                372,
                375,
                382,
                383,
                384,
                385,
                386,
                387,
                388,
                389,
                390,
                391,
                392,
                393,
                395,
                396,
                397,
                399,
                402,
                404,
                413,
                417,
                418,
            ]
        elif self.name == "prison_term":
            return None
        else:
            assert "Dataset name {} does not exist".format(self.name)



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""COTE: Chinese Opinion Target Extraction."""

import csv
import os
import textwrap
import numpy as np

import datasets

logger = datasets.logging.get_logger(__name__)

_CITATION = """\
@inproceedings{li2018character,
  title={Character-based bilstm-crf incorporating pos and dictionaries for chinese opinion target extraction},
  author={Li, Yanzeng and Liu, Tingwen and Li, Diying and Li, Quangang and Shi, Jinqiao and Wang, Yanqiu},
  booktitle={Asian Conference on Machine Learning},
  pages={518--533},
  year={2018},
  organization={PMLR}
}
"""

_DESCRIPTION = """\
COTE, a dataset for Opinion target extraction (OTE) for sentiment analysis, which aims to extract target of a given text. This dataset covers data crawled on Baidu, Dianping, and Mafengwo.
More information refer to https://www.luge.ai/#/luge/dataDetail?id=19.
"""

_COTE_URLs = {
    # pylint: disable=line-too-long
    "bd": "https://paddlenlp.bj.bcebos.com/datasets/COTE-BD.zip",
    "mfw": "https://paddlenlp.bj.bcebos.com/datasets/COTE-MFW.zip",
    "dp": "https://paddlenlp.bj.bcebos.com/datasets/COTE-DP.zip",
    # pylint: enable=line-too-long
}


class COTEConfig(datasets.BuilderConfig):
    """BuilderConfig for COTE."""

    def __init__(self, data_url=None, data_dir=None, **kwargs):
        """BuilderConfig for COTE.

        Args:
          data_url: `string`, url to download the zip file.
          data_dir: `string`, the path to the folder containing the tsv files in the downloaded zip.
          **kwargs: keyword arguments forwarded to super.
        """
        super(COTEConfig, self).__init__(**kwargs)
        self.data_url = data_url
        self.data_dir = data_dir


class COTE(datasets.GeneratorBasedBuilder):
    """COTE: Chinese Opinion Target Extraction."""

    BUILDER_CONFIGS = [
        COTEConfig(
            name="bd",
            data_url=_COTE_URLs["bd"],
            data_dir="COTE-BD",
            version=datasets.Version("1.0.0", ""),
            description="COTE-BD crawled on baidu.",
        ),
        COTEConfig(
            name="mfw",
            data_url=_COTE_URLs["mfw"],
            data_dir="COTE-MFW",
            version=datasets.Version("1.0.0", ""),
            description="COTE-MFW crawled on Mafengwo.",
        ),
        COTEConfig(
            name="dp",
            data_url=_COTE_URLs["dp"],
            data_dir="COTE-DP",
            version=datasets.Version("1.0.0", ""),
            description="COTE-DP crawled on Dianping.",
        ),
    ]

    def _info(self):
        features = {
            "id": datasets.Value("int32"),
            "text_a": datasets.Value("string"),
            "label": datasets.Value("string"),
        }

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(features),
            homepage="https://www.luge.ai/#/luge/dataDetail?id=19",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        downloaded_dir = dl_manager.download_and_extract(self.config.data_url)
        data_dir = os.path.join(downloaded_dir, self.config.data_dir)

        train_split = datasets.SplitGenerator(
            name=datasets.Split.TRAIN, gen_kwargs={"filepath": os.path.join(data_dir, "train.tsv"), "split": "train"}
        )
        test_split = datasets.SplitGenerator(
            name=datasets.Split.TEST, gen_kwargs={"filepath": os.path.join(data_dir, "test.tsv"), "split": "test"}
        )

        return [train_split, test_split]

    def _generate_examples(self, filepath, split):
        """This function returns the examples in the raw (text) form."""
        logger.info("generating examples from = %s", filepath)

        with open(filepath, encoding="utf8") as f:
            reader = csv.DictReader(f, delimiter="\t", quoting=csv.QUOTE_NONE)

            for idx, row in enumerate(reader):
                example = {}
                example["id"] = idx
                example["text_a"] = row["text_a"]

                if split == "train":
                    example["label"] = row["label"]
                else:
                    example["label"] = ""

                # Filter out corrupted rows.
                for value in example.values():
                    if value is None:
                        break
                else:
                    yield idx, example



# Copyright (c) 2021 PaddlePaddle Authors. All Rights Reserved.
# Copyright 2021 The HuggingFace Inc. team.
#
# Licensed under the Apache License, Version 2.0 (the "License"
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import math

import paddle
import paddle.nn as nn
import paddle.nn.functional as F

from .. import PretrainedModel, register_base_model

__all__ = [
    "MobileBertModel",
    "MobileBertPretrainedModel",
    "MobileBertForPreTraining",
    "MobileBertForSequenceClassification",
    "MobileBertForQuestionAnswering",
]

ACT2FN = {
    "relu": F.relu,
    "gelu": F.gelu,
}


class NoNorm(nn.Layer):
    def __init__(self, feat_size, eps=None):
        super().__init__()
        if isinstance(feat_size, int):
            feat_size = [feat_size]
        self.bias = paddle.create_parameter(feat_size, "float32", is_bias=True)
        self.weight = paddle.create_parameter(
            feat_size, "float32", default_initializer=paddle.nn.initializer.Constant(value=1.0)
        )

    def forward(self, input_tensor):
        return input_tensor * self.weight + self.bias


NORM2FN = {"layer_norm": nn.LayerNorm, "no_norm": NoNorm}


class MobileBertEmbeddings(nn.Layer):
    """Construct the embeddings from word, position and token_type embeddings."""

    def __init__(
        self,
        vocab_size,
        embedding_size=128,
        hidden_size=512,
        hidden_dropout_prob=0.0,
        max_position_embeddings=512,
        type_vocab_size=2,
        layer_norm_eps=1e-12,
        pad_token_id=1,
        trigram_input=True,
        normalization_type="no_norm",
    ):
        super().__init__()
        self.trigram_input = trigram_input
        self.embedding_size = embedding_size
        self.hidden_size = hidden_size
        self.word_embeddings = nn.Embedding(vocab_size, embedding_size, padding_idx=pad_token_id)
        self.position_embeddings = nn.Embedding(max_position_embeddings, hidden_size)
        self.token_type_embeddings = nn.Embedding(type_vocab_size, hidden_size)

        embed_dim_multiplier = 3 if self.trigram_input else 1
        embedded_input_size = self.embedding_size * embed_dim_multiplier
        self.embedding_transformation = nn.Linear(embedded_input_size, hidden_size)

        self.layer_norm = NORM2FN[normalization_type](hidden_size)
        self.dropout = nn.Dropout(hidden_dropout_prob)

        # position_ids (1, len position emb) is contiguous in memory and exported when serialized
        self.register_buffer("position_ids", paddle.arange(max_position_embeddings).expand((1, -1)))

    def forward(self, input_ids=None, token_type_ids=None, position_ids=None, inputs_embeds=None):
        if input_ids is not None:
            input_shape = input_ids.shape
        else:
            input_shape = inputs_embeds.shape[:-1]

        seq_length = input_shape[1]

        if position_ids is None:
            position_ids = self.position_ids[:, :seq_length]

        if token_type_ids is None:
            token_type_ids = paddle.zeros(input_shape, dtype="int64")
        if inputs_embeds is None:
            inputs_embeds = self.word_embeddings(input_ids)

        if self.trigram_input:
            # From the paper MobileBERT: a Compact Task-Agnostic BERT for Resource-Limited
            # Devices (https://arxiv.org/abs/2004.02984)
            #
            # The embedding table in BERT models accounts for a substantial proportion of model size. To compress
            # the embedding layer, we reduce the embedding dimension to 128 in MobileBERT.
            # Then, we apply a 1D convolution with kernel size 3 on the raw token embedding to produce a 512
            # dimensional output.
            inputs_embeds = paddle.concat(
                [
                    nn.functional.pad(inputs_embeds[:, 1:], [0, 0, 0, 1, 0, 0], value=0),
                    inputs_embeds,
                    nn.functional.pad(inputs_embeds[:, :-1], [0, 0, 1, 0, 0, 0], value=0),
                ],
                axis=2,
            )
        if self.trigram_input or self.embedding_size != self.hidden_size:
            inputs_embeds = self.embedding_transformation(inputs_embeds)

        # Add positional embeddings and token type embeddings, then layer
        # normalize and perform dropout.
        position_embeddings = self.position_embeddings(position_ids)
        token_type_embeddings = self.token_type_embeddings(token_type_ids)
        embeddings = inputs_embeds + position_embeddings + token_type_embeddings
        embeddings = self.layer_norm(embeddings)
        embeddings = self.dropout(embeddings)
        return embeddings


class MobileBertAttention(nn.Layer):
    def __init__(
        self,
        num_attention_heads=4,
        true_hidden_size=128,
        hidden_size=512,
        use_bottleneck_attention=False,
        attention_probs_dropout_prob=0.1,
        use_bottleneck=True,
        normalization_type="no_norm",
        layer_norm_eps=1e-12,
        hidden_dropout_prob=0.0,
    ):
        super().__init__()

        self.num_attention_heads = num_attention_heads
        self.attention_head_size = int(true_hidden_size / num_attention_heads)
        self.all_head_size = self.num_attention_heads * self.attention_head_size
        self.query = nn.Linear(true_hidden_size, self.all_head_size)
        self.key = nn.Linear(true_hidden_size, self.all_head_size)
        self.value = nn.Linear(true_hidden_size if use_bottleneck_attention else hidden_size, self.all_head_size)

        self.attention_dropout = nn.Dropout(attention_probs_dropout_prob)

        self.use_bottleneck = use_bottleneck
        self.dense = nn.Linear(true_hidden_size, true_hidden_size)
        self.layer_norm = NORM2FN[normalization_type](true_hidden_size, eps=layer_norm_eps)
        if not self.use_bottleneck:
            self.output_dropout = nn.Dropout(hidden_dropout_prob)

    def transpose_for_scores(self, x):
        new_x_shape = x.shape[:-1] + [self.num_attention_heads, self.attention_head_size]
        x = x.reshape(new_x_shape)
        return x.transpose(perm=(0, 2, 1, 3))

    def forward(
        self,
        query_tensor,
        key_tensor,
        value_tensor,
        hidden_states,
        attention_mask=None,
        head_mask=None,
        output_attentions=None,
    ):

        mixed_query_layer = self.query(query_tensor)
        mixed_key_layer = self.key(key_tensor)
        mixed_value_layer = self.value(value_tensor)

        query_layer = self.transpose_for_scores(mixed_query_layer)
        key_layer = self.transpose_for_scores(mixed_key_layer)
        value_layer = self.transpose_for_scores(mixed_value_layer)

        # Take the dot product between "query" and "key" to get the raw attention scores.
        attention_scores = paddle.matmul(query_layer, key_layer, transpose_y=True)
        attention_scores = attention_scores / math.sqrt(self.attention_head_size)
        if attention_mask is not None:
            # Apply the attention mask is (precomputed for all layers in BertModel forward() function)
            attention_scores = attention_scores + attention_mask
        # Normalize the attention scores to probabilities.
        attention_probs = nn.Softmax(axis=-1)(attention_scores)
        # This is actually dropping out entire tokens to attend to, which might
        # seem a bit unusual, but is taken from the original Transformer paper.
        attention_probs = self.attention_dropout(attention_probs)
        # Mask heads if we want to
        if head_mask is not None:
            attention_probs = attention_probs * head_mask
        context_layer = paddle.matmul(attention_probs, value_layer)
        context_layer = context_layer.transpose(perm=(0, 2, 1, 3))
        new_context_layer_shape = context_layer.shape[:-2] + [self.all_head_size]
        context_layer = context_layer.reshape(new_context_layer_shape)

        # dense layer shape to be checked
        projected_context_layer = self.dense(context_layer)

        # Run a linear projection of `hidden_size` then add a residual
        # with `hidden_states`.
        if not self.use_bottleneck:
            projected_context_layer = self.output_dropout(projected_context_layer)
        layer_normed_context_layer = self.layer_norm(hidden_states + projected_context_layer)

        outputs = (layer_normed_context_layer, attention_probs) if output_attentions else (layer_normed_context_layer,)
        return outputs


class MobileBertIntermediate(nn.Layer):
    def __init__(
        self,
        true_hidden_size=128,
        intermediate_size=512,
        hidden_act="relu",
    ):
        super().__init__()
        self.dense = nn.Linear(true_hidden_size, intermediate_size)
        if isinstance(hidden_act, str):
            self.intermediate_act_fn = ACT2FN[hidden_act]
        else:
            self.intermediate_act_fn = hidden_act

    def forward(self, hidden_states):
        hidden_states = self.dense(hidden_states)
        hidden_states = self.intermediate_act_fn(hidden_states)
        return hidden_states


class OutputBottleneck(nn.Layer):
    def __init__(
        self,
        true_hidden_size=128,
        hidden_size=512,
        normalization_type="no_norm",
        layer_norm_eps=1e-12,
        hidden_dropout_prob=0.0,
    ):
        super().__init__()
        self.dense = nn.Linear(true_hidden_size, hidden_size)
        self.layer_norm = NORM2FN[normalization_type](hidden_size, eps=layer_norm_eps)
        self.dropout = nn.Dropout(hidden_dropout_prob)

    def forward(self, hidden_states, residual_tensor):
        layer_outputs = self.dense(hidden_states)
        layer_outputs = self.dropout(layer_outputs)
        layer_outputs = self.layer_norm(layer_outputs + residual_tensor)
        return layer_outputs


class MobileBertOutput(nn.Layer):
    def __init__(
        self,
        use_bottleneck=True,
        intermediate_size=512,
        true_hidden_size=128,
        hidden_size=512,
        normalization_type="no_norm",
        hidden_dropout_prob=0.0,
        layer_norm_eps=1e-12,
    ):
        super().__init__()
        self.use_bottleneck = use_bottleneck
        self.dense = nn.Linear(intermediate_size, true_hidden_size)
        self.layer_norm = NORM2FN[normalization_type](true_hidden_size)
        if not self.use_bottleneck:
            self.dropout = nn.Dropout(hidden_dropout_prob)
        else:
            self.bottleneck = OutputBottleneck(
                true_hidden_size=true_hidden_size,
                hidden_size=hidden_size,
                normalization_type=normalization_type,
                layer_norm_eps=layer_norm_eps,
                hidden_dropout_prob=hidden_dropout_prob,
            )

    def forward(self, intermediate_states, residual_tensor_1, residual_tensor_2):
        layer_output = self.dense(intermediate_states)
        if not self.use_bottleneck:
            layer_output = self.dropout(layer_output)
            layer_output = self.layer_norm(layer_output + residual_tensor_1)
        else:
            layer_output = self.layer_norm(layer_output + residual_tensor_1)
            layer_output = self.bottleneck(layer_output, residual_tensor_2)
        return layer_output


class BottleneckLayer(nn.Layer):
    def __init__(
        self,
        hidden_size=512,
        intra_bottleneck_size=128,
        normalization_type="no_norm",
        layer_norm_eps=1e-12,
    ):
        super().__init__()
        self.dense = nn.Linear(hidden_size, intra_bottleneck_size)
        self.layer_norm = NORM2FN[normalization_type](intra_bottleneck_size, eps=layer_norm_eps)

    def forward(self, hidden_states):
        layer_input = self.dense(hidden_states)
        layer_input = self.layer_norm(layer_input)
        return layer_input


class Bottleneck(nn.Layer):
    def __init__(
        self,
        key_query_shared_bottleneck=True,
        use_bottleneck_attention=False,
        hidden_size=512,
        intra_bottleneck_size=128,
        normalization_type="no_norm",
        layer_norm_eps=1e-12,
    ):
        super().__init__()
        self.key_query_shared_bottleneck = key_query_shared_bottleneck
        self.use_bottleneck_attention = use_bottleneck_attention
        self.input = BottleneckLayer(
            hidden_size=hidden_size,
            intra_bottleneck_size=intra_bottleneck_size,
            normalization_type=normalization_type,
            layer_norm_eps=layer_norm_eps,
        )
        if self.key_query_shared_bottleneck:
            self.attention = BottleneckLayer(
                hidden_size=hidden_size,
                intra_bottleneck_size=intra_bottleneck_size,
                normalization_type=normalization_type,
                layer_norm_eps=layer_norm_eps,
            )

    def forward(self, hidden_states):
        # This method can return three different tuples of values. These different values make use of bottlenecks,
        # which are linear layers used to project the hidden states to a lower-dimensional vector, reducing memory
        # usage. These linear layer have weights that are learned during training.
        #
        # If `config.use_bottleneck_attention`, it will return the result of the bottleneck layer four times for the
        # key, query, value, and "layer input" to be used by the attention layer.
        # This bottleneck is used to project the hidden. This last layer input will be used as a residual tensor
        # in the attention self output, after the attention scores have been computed.
        #
        # If not `config.use_bottleneck_attention` and `config.key_query_shared_bottleneck`, this will return
        # four values, three of which have been passed through a bottleneck: the query and key, passed through the same
        # bottleneck, and the residual layer to be applied in the attention self output, through another bottleneck.
        #
        # Finally, in the last case, the values for the query, key and values are the hidden states without bottleneck,
        # and the residual layer will be this value passed through a bottleneck.

        bottlenecked_hidden_states = self.input(hidden_states)
        if self.use_bottleneck_attention:
            return (bottlenecked_hidden_states,) * 4
        elif self.key_query_shared_bottleneck:
            shared_attention_input = self.attention(hidden_states)
            return (shared_attention_input, shared_attention_input, hidden_states, bottlenecked_hidden_states)
        else:
            return (hidden_states, hidden_states, hidden_states, bottlenecked_hidden_states)


class FFNOutput(nn.Layer):
    def __init__(
        self,
        intermediate_size=512,
        true_hidden_size=128,
        normalization_type="no_norm",
        layer_norm_eps=1e-12,
    ):
        super().__init__()
        self.dense = nn.Linear(intermediate_size, true_hidden_size)
        self.layer_norm = NORM2FN[normalization_type](true_hidden_size, eps=layer_norm_eps)

    def forward(self, hidden_states, residual_tensor):
        layer_outputs = self.dense(hidden_states)
        layer_outputs = self.layer_norm(layer_outputs + residual_tensor)
        return layer_outputs


class FFNLayer(nn.Layer):
    def __init__(
        self,
        true_hidden_size=128,
        intermediate_size=512,
        hidden_act="relu",
        normalization_type="no_norm",
        layer_norm_eps=1e-12,
    ):
        super().__init__()
        self.intermediate = MobileBertIntermediate(
            true_hidden_size=true_hidden_size,
            intermediate_size=intermediate_size,
            hidden_act=hidden_act,
        )
        self.output = FFNOutput(
            intermediate_size=intermediate_size,
            true_hidden_size=true_hidden_size,
            normalization_type=normalization_type,
            layer_norm_eps=layer_norm_eps,
        )

    def forward(self, hidden_states):
        intermediate_output = self.intermediate(hidden_states)
        layer_outputs = self.output(intermediate_output, hidden_states)
        return layer_outputs


class MobileBertLayer(nn.Layer):
    def __init__(
        self,
        use_bottleneck=True,
        num_feedforward_networks=4,
        num_attention_heads=4,
        true_hidden_size=128,
        use_bottleneck_attention=False,
        attention_probs_dropout_prob=0.1,
        normalization_type="no_norm",
        layer_norm_eps=1e-12,
        hidden_dropout_prob=0.0,
        intermediate_size=512,
        hidden_act="relu",
        hidden_size=512,
        key_query_shared_bottleneck=True,
        intra_bottleneck_size=128,
    ):
        super().__init__()
        self.use_bottleneck = use_bottleneck
        self.num_feedforward_networks = num_feedforward_networks

        self.attention = MobileBertAttention(
            num_attention_heads=num_attention_heads,
            true_hidden_size=true_hidden_size,
            hidden_size=hidden_size,
            use_bottleneck_attention=use_bottleneck_attention,
            attention_probs_dropout_prob=attention_probs_dropout_prob,
            use_bottleneck=use_bottleneck,
            normalization_type=normalization_type,
            layer_norm_eps=layer_norm_eps,
            hidden_dropout_prob=hidden_dropout_prob,
        )
        self.intermediate = MobileBertIntermediate(
            true_hidden_size=true_hidden_size,
            intermediate_size=intermediate_size,
            hidden_act=hidden_act,
        )
        self.output = MobileBertOutput(
            use_bottleneck=use_bottleneck,
            intermediate_size=intermediate_size,
            true_hidden_size=true_hidden_size,
            hidden_size=hidden_size,
            normalization_type=normalization_type,
            hidden_dropout_prob=hidden_dropout_prob,
            layer_norm_eps=layer_norm_eps,
        )
        if self.use_bottleneck:
            self.bottleneck = Bottleneck(
                key_query_shared_bottleneck=key_query_shared_bottleneck,
                use_bottleneck_attention=use_bottleneck_attention,
                hidden_size=hidden_size,
                intra_bottleneck_size=intra_bottleneck_size,
                normalization_type=normalization_type,
                layer_norm_eps=layer_norm_eps,
            )
        if num_feedforward_networks > 1:
            self.ffn = nn.LayerList(
                [
                    FFNLayer(
                        true_hidden_size=true_hidden_size,
                        intermediate_size=intermediate_size,
                        hidden_act=hidden_act,
                        normalization_type=normalization_type,
                        layer_norm_eps=layer_norm_eps,
                    )
                    for _ in range(num_feedforward_networks - 1)
                ]
            )

    def forward(
        self,
        hidden_states,
        attention_mask=None,
        head_mask=None,
        output_attentions=None,
    ):
        if self.use_bottleneck:
            query_tensor, key_tensor, value_tensor, layer_input = self.bottleneck(hidden_states)
        else:
            query_tensor, key_tensor, value_tensor, layer_input = [hidden_states] * 4

        self_attention_outputs = self.attention(
            query_tensor,
            key_tensor,
            value_tensor,
            layer_input,
            attention_mask,
            head_mask,
            output_attentions=output_attentions,
        )
        attention_output = self_attention_outputs[0]
        s = (attention_output,)
        outputs = self_attention_outputs[1:]  # add self attentions if we output attention weights

        if self.num_feedforward_networks != 1:
            for i, ffn_module in enumerate(self.ffn):
                attention_output = ffn_module(attention_output)
                s += (attention_output,)

        intermediate_output = self.intermediate(attention_output)
        layer_output = self.output(intermediate_output, attention_output, hidden_states)
        outputs = (
            (layer_output,)
            + outputs
            + (
                paddle.to_tensor(1000),
                query_tensor,
                key_tensor,
                value_tensor,
                layer_input,
                attention_output,
                intermediate_output,
            )
            + s
        )
        return outputs


class MobileBertEncoder(nn.Layer):
    def __init__(
        self,
        num_hidden_layers=24,
        use_bottleneck=True,
        num_feedforward_networks=4,
        num_attention_heads=4,
        true_hidden_size=128,
        use_bottleneck_attention=False,
        attention_probs_dropout_prob=0.1,
        normalization_type="no_norm",
        layer_norm_eps=1e-12,
        hidden_dropout_prob=0.0,
        intermediate_size=512,
        hidden_act="relu",
        hidden_size=512,
        key_query_shared_bottleneck=True,
    ):
        super().__init__()
        self.layers = nn.LayerList(
            [
                MobileBertLayer(
                    use_bottleneck=use_bottleneck,
                    num_feedforward_networks=num_feedforward_networks,
                    num_attention_heads=num_attention_heads,
                    true_hidden_size=true_hidden_size,
                    use_bottleneck_attention=use_bottleneck_attention,
                    attention_probs_dropout_prob=attention_probs_dropout_prob,
                    normalization_type=normalization_type,
                    layer_norm_eps=layer_norm_eps,
                    hidden_dropout_prob=hidden_dropout_prob,
                    intermediate_size=intermediate_size,
                    key_query_shared_bottleneck=key_query_shared_bottleneck,
                    hidden_act=hidden_act,
                    hidden_size=hidden_size,
                )
                for _ in range(num_hidden_layers)
            ]
        )

    def forward(
        self,
        hidden_states,
        attention_mask=None,
        head_mask=None,
        output_attentions=False,
        output_hidden_states=False,
    ):
        all_hidden_states = () if output_hidden_states else None
        all_attentions = () if output_attentions else None
        for i, layer_module in enumerate(self.layers):
            if output_hidden_states:
                all_hidden_states = all_hidden_states + (hidden_states,)

            layer_outputs = layer_module(
                hidden_states,
                attention_mask,
                head_mask[i],
                output_attentions,
            )
            hidden_states = layer_outputs[0]

            if output_attentions:
                all_attentions = all_attentions + (layer_outputs[1],)

        # Add last layer
        if output_hidden_states:
            all_hidden_states = all_hidden_states + (hidden_states,)

        return tuple(v for v in [hidden_states, all_hidden_states, all_attentions] if v is not None)


class MobileBertPooler(nn.Layer):
    def __init__(
        self,
        classifier_activation=False,
        hidden_size=512,
    ):
        super().__init__()
        self.do_activate = classifier_activation
        if self.do_activate:
            self.dense = nn.Linear(hidden_size, hidden_size)

    def forward(self, hidden_states):
        # We "pool" the model by simply taking the hidden state corresponding
        # to the first token.
        first_token_tensor = hidden_states[:, 0]
        if not self.do_activate:
            return first_token_tensor
        else:
            pooled_output = self.dense(first_token_tensor)
            pooled_output = paddle.tanh(pooled_output)
            return pooled_output


class MobileBertPredictionHeadTransform(nn.Layer):
    def __init__(
        self,
        hidden_size=512,
        hidden_act="relu",
        layer_norm_eps=1e-12,
    ):
        super().__init__()
        self.dense = nn.Linear(hidden_size, hidden_size)
        if isinstance(hidden_act, str):
            self.transform_act_fn = ACT2FN[hidden_act]
        else:
            self.transform_act_fn = hidden_act
        self.layer_norm = NORM2FN["layer_norm"](hidden_size, epsilon=layer_norm_eps)

    def forward(self, hidden_states):
        hidden_states = self.dense(hidden_states)
        hidden_states = self.transform_act_fn(hidden_states)
        hidden_states = self.layer_norm(hidden_states)
        return hidden_states


class MobileBertLMPredictionHead(nn.Layer):
    def __init__(
        self,
        vocab_size=30522,
        hidden_size=512,
        embedding_size=128,
        hidden_act="relu",
        layer_norm_eps=1e-12,
    ):
        super().__init__()
        self.transform = MobileBertPredictionHeadTransform(
            hidden_size=hidden_size,
            hidden_act=hidden_act,
            layer_norm_eps=layer_norm_eps,
        )
        # The output weights are the same as the input embeddings, but there is
        # an output-only bias for each token.
        self.dense = nn.Linear(vocab_size, hidden_size - embedding_size, bias_attr=False)
        self.decoder = nn.Linear(embedding_size, vocab_size)

    def forward(self, hidden_states):
        hidden_states = self.transform(hidden_states)
        param_concat = paddle.concat([self.decoder.weight, self.dense.weight.t()], axis=0)

        hidden_states = paddle.matmul(hidden_states, param_concat)
        hidden_states += self.decoder.bias
        return hidden_states


class MobileBertOnlyMLMHead(nn.Layer):
    def __init__(
        self,
        vocab_size=30522,
        hidden_size=512,
        embedding_size=128,
        hidden_act="relu",
        layer_norm_eps=1e-12,
    ):
        super().__init__()
        self.predictions = MobileBertLMPredictionHead(
            vocab_size=vocab_size,
            hidden_size=hidden_size,
            embedding_size=embedding_size,
            hidden_act=hidden_act,
            layer_norm_eps=layer_norm_eps,
        )

    def forward(self, sequence_output):
        prediction_scores = self.predictions(sequence_output)
        return prediction_scores


class MobileBertPreTrainingHeads(nn.Layer):
    def __init__(
        self,
        vocab_size=30522,
        hidden_size=512,
        embedding_size=128,
        hidden_act="relu",
        layer_norm_eps=1e-12,
    ):
        super().__init__()
        self.predictions = MobileBertLMPredictionHead(
            vocab_size=vocab_size,
            hidden_size=hidden_size,
            embedding_size=embedding_size,
            hidden_act=hidden_act,
            layer_norm_eps=layer_norm_eps,
        )
        self.seq_relationship = nn.Linear(hidden_size, 2)

    def forward(self, sequence_output, pooled_output):
        prediction_scores = self.predictions(sequence_output)
        seq_relationship_score = self.seq_relationship(pooled_output)
        return prediction_scores, seq_relationship_score


class MobileBertPretrainedModel(PretrainedModel):
    """
    An abstract class for pretrained MobileBert models. It provides MobileBert related
    `model_config_file`, `resource_files_names`, `pretrained_resource_files_map`,
    `pretrained_init_configuration`, `base_model_prefix` for downloading and
    loading pretrained models.
    See :class:`~paddlenlp.transformers.model_utils.PretrainedModel` for more details.
    """

    pretrained_init_configuration = {
        "mobilebert-uncased": {
            "attention_probs_dropout_prob": 0.1,
            "classifier_activation": False,
            "embedding_size": 128,
            "hidden_act": "relu",
            "hidden_dropout_prob": 0.0,
            "hidden_size": 512,
            "initializer_range": 0.02,
            "intermediate_size": 512,
            "intra_bottleneck_size": 128,
            "key_query_shared_bottleneck": True,
            "layer_norm_eps": 1e-12,
            "max_position_embeddings": 512,
            "normalization_type": "no_norm",
            "num_attention_heads": 4,
            "num_feedforward_networks": 4,
            "num_hidden_layers": 24,
            "pad_token_id": 0,
            "trigram_input": True,
            "true_hidden_size": 128,
            "type_vocab_size": 2,
            "use_bottleneck": True,
            "use_bottleneck_attention": False,
            "vocab_size": 30522,
        }
    }

    pretrained_resource_files_map = {
        "model_state": {
            "mobilebert-uncased": "https://bj.bcebos.com/paddlenlp/models/transformers/mobilebert/mobilebert-uncased/model_state.pdparams"
        }
    }
    base_model_prefix = "mobilebert"

    def init_weights(self):
        # Initialize the weights.
        self.apply(self._init_weights)

    def _init_weights(self, layer):
        # Initialize the weights.
        if isinstance(layer, nn.Linear):
            # In the dygraph mode, use the `set_value` to reset the parameter directly,
            # and reset the `state_dict` to update parameter in static mode.
            layer.weight.set_value(
                paddle.tensor.normal(
                    mean=0.0,
                    std=self.initializer_range
                    if hasattr(self, "initializer_range")
                    else self.mobilebert.config["initializer_range"],
                    shape=layer.weight.shape,
                )
            )
            if layer.bias is not None:
                layer.bias.set_value(paddle.zeros_like(layer.bias))
        elif isinstance(layer, (nn.LayerNorm, NoNorm)):
            layer.bias.set_value(paddle.zeros_like(layer.bias))
            layer.weight.set_value(paddle.ones_like(layer.weight))


class MobileBertForPreTraining(MobileBertPretrainedModel):
    """
    MobileBert Model with pretraining tasks on top.

    Args:
        bert (:class:`MobileBertModel`):
            An instance of :class:`MobileBertModel`.
    """

    def __init__(self, mobilebert):
        super(MobileBertForPreTraining, self).__init__()
        self.mobilebert = mobilebert
        self.cls = MobileBertPreTrainingHeads(
            self.mobilebert.config["vocab_size"],
            self.mobilebert.config["hidden_size"],
            self.mobilebert.config["embedding_size"],
            self.mobilebert.config["hidden_act"],
            self.mobilebert.config["layer_norm_eps"],
        )

        self.init_weights()

    def get_output_embeddings(self):
        return self.cls.predictions.decoder

    def set_output_embeddings(self, new_embeddigs):
        self.cls.predictions.decoder = new_embeddigs

    def forward(
        self,
        input_ids=None,
        attention_mask=None,
        token_type_ids=None,
        position_ids=None,
        head_mask=None,
        inputs_embeds=None,
        output_attentions=None,
        output_hidden_states=None,
    ):
        r"""
        The MobileBertForPreTraining forward method, overrides the __call__() special method.

        Args:
            input_ids (Tensor):
                See :class:`MobileBertModel`.
            token_type_ids (Tensor, optional):
                See :class:`MobileBertModel`.
            position_ids(Tensor, optional):
                See :class:`MobileBertModel`.
            head_mask (Tensor, optional):
                See :class:`MobileBertModel`.
            attention_mask (Tensor, optional):
                See :class:`MobileBertModel`.
            inputs_embeds (Tensor, optional):
                See :class:`MobileBertModel`.
            output_attentions (bool, optional):
                See :class:`MobileBertModel`.
            output_hidden_states (bool, optional):
                See :class:`MobileBertModel`.

        Returns:
            tuple: Returns tuple (``prediction_scores``, ``seq_relationship_score``).
            With the fields:
            - `prediction_scores` (Tensor):
                The scores of masked token prediction. Its data type should be float32.
                If `masked_positions` is None, its shape is [batch_size, sequence_length, vocab_size].
                Otherwise, its shape is [batch_size, mask_token_num, vocab_size].
            - `seq_relationship_score` (Tensor):
                The scores of next sentence prediction.
                Its data type should be float32 and its shape is [batch_size, 2].

        .. code-block::
                import paddle
                from paddlenlp.transformers import MobileBertModel, MobileBertTokenizer
                tokenizer = MobileBertTokenizer.from_pretrained('mobilebert-uncased')
                model = MobileBertForPreTraining.from_pretrained('mobilebert-uncased')
                inputs = tokenizer("Welcome to use PaddlePaddle and PaddleNLP!")
                inputs = {k:paddle.to_tensor([v]) for (k, v) in inputs.items()}
                output = model(**inputs)
                prediction_logits = outputs[0]
                seq_relationship_logits = outputs[1]
        """

        outputs = self.mobilebert(
            input_ids,
            attention_mask=attention_mask,
            token_type_ids=token_type_ids,
            position_ids=position_ids,
            head_mask=head_mask,
            inputs_embeds=inputs_embeds,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
        )
        sequence_output, pooled_output = outputs[:2]
        prediction_scores, seq_relationship_score = self.cls(sequence_output, pooled_output)

        output = (prediction_scores, seq_relationship_score) + outputs[2:]
        return output


@register_base_model
class MobileBertModel(MobileBertPretrainedModel):
    """
    The bare MobileBert Model transformer outputting raw hidden-states.
    This model inherits from :class:`~paddlenlp.transformers.model_utils.PretrainedModel`.
    Refer to the superclass documentation for the generic methods.
    This model is also a Paddle `paddle.nn.Layer <https://www.paddlepaddle.org.cn/documentation
    /docs/en/api/paddle/fluid/dygraph/layers/Layer_en.html>`__ subclass. Use it as a regular Paddle Layer
    and refer to the Paddle documentation for all matter related to general usage and behavior.

    Args:
        vocab_size (int):
            Vocabulary size of `inputs_ids` in `MobileBertModel`. Also is the vocab size of token embedding matrix.
            Defines the number of different tokens that can be represented by the `inputs_ids` passed when calling `MobileBertModel`.
        embedding_size (int, optional):
            Embedding dimensionality of lookup_table in the embedding layer. Defaults to `128`.
        hidden_size (int, optional):
            Dimensionality of the embedding layer, encoder layer and pooler layer. Defaults to `512`.
        true_hidden_size (int, optional):
            Dimensionality of input_tensor in self attention layer. Defaults to `128`.
        use_bottleneck_attention (bool, optional):
            Using bottleneck to value tensor in self attention layer. Defaults to `False`.
        key_query_shared_bottleneck (bool, optional):
            Key and query shared bottleneck layer. Defaults to `True`.
        num_hidden_layers (int, optional):
            Number of hidden layers in the Transformer encoder. Defaults to `24`.
        num_attention_heads (int, optional):
            Number of attention heads for each attention layer in the Transformer encoder.
            Defaults to `4`.
        intermediate_size (int, optional):
            Dimensionality of the feed-forward (ff) layer in the encoder. Input tensors
            to ff layers are firstly projected from `hidden_size` to `intermediate_size`,
            and then projected back to `hidden_size`. Typically `intermediate_size` is larger than `hidden_size`.
            Defaults to `512`.
        hidden_act (str, optional):
            The non-linear activation function in the feed-forward layer.
            ``"gelu"``, ``"relu"`` and any other paddle supported activation functions
            are supported. Defaults to `"relu"`.
        hidden_dropout_prob (float, optional):
            The dropout probability for all fully connected layers in the embeddings and encoder.
            Defaults to `0.1`.
        attention_probs_dropout_prob (float, optional):
            The dropout probability used in MultiHeadAttention in all encoder layers to drop some attention target.
            Defaults to `0.1`.
        max_position_embeddings (int, optional):
            The maximum value of the dimensionality of position encoding, which dictates the maximum supported length of an input
            sequence. Defaults to `512`.
        type_vocab_size (int, optional):
            The vocabulary size of `token_type_ids`.
            Defaults to `2`.
        initializer_range (float, optional):
            The standard deviation of the normal initializer.
            Defaults to 0.02.
            .. note::
                A normal_initializer initializes weight matrices as normal distributions.
                See :meth:`MobileBertPretrainedModel.init_weights()` for how weights are initialized in `MobileBertModel`.
        pad_token_id (int, optional):
            The index of padding token in the token vocabulary.
            Defaults to `1`.
        add_pooling_layer (bool, optional):
            Adding the pooling Layer after the encoder layer. Defaults to `True`.
        classifier_activation (bool, optional):
            Using the non-linear activation function in the pooling layer. Defaults to `False`.

    """

    def __init__(
        self,
        vocab_size,
        embedding_size=128,
        hidden_size=512,
        hidden_dropout_prob=0.0,
        max_position_embeddings=512,
        type_vocab_size=2,
        layer_norm_eps=1e-12,
        pad_token_id=1,
        trigram_input=True,
        normalization_type="no_norm",
        num_hidden_layers=24,
        use_bottleneck=True,
        num_feedforward_networks=4,
        num_attention_heads=4,
        true_hidden_size=128,
        use_bottleneck_attention=False,
        attention_probs_dropout_prob=0.1,
        intermediate_size=512,
        intra_bottleneck_size=128,
        hidden_act="relu",
        classifier_activation=False,
        initializer_range=0.02,
        key_query_shared_bottleneck=True,
        add_pooling_layer=True,
    ):
        super(MobileBertModel, self).__init__()

        self.initializer_range = initializer_range
        if use_bottleneck:
            true_hidden_size = intra_bottleneck_size
        else:
            true_hidden_size = hidden_size
        self.embeddings = MobileBertEmbeddings(
            vocab_size=vocab_size,
            embedding_size=embedding_size,
            hidden_size=hidden_size,
            hidden_dropout_prob=hidden_dropout_prob,
            max_position_embeddings=max_position_embeddings,
            type_vocab_size=type_vocab_size,
            layer_norm_eps=layer_norm_eps,
            pad_token_id=pad_token_id,
            trigram_input=trigram_input,
            normalization_type=normalization_type,
        )
        self.encoder = MobileBertEncoder(
            num_hidden_layers=num_hidden_layers,
            use_bottleneck=use_bottleneck,
            num_feedforward_networks=num_feedforward_networks,
            num_attention_heads=num_attention_heads,
            true_hidden_size=true_hidden_size,
            use_bottleneck_attention=use_bottleneck_attention,
            attention_probs_dropout_prob=attention_probs_dropout_prob,
            normalization_type=normalization_type,
            layer_norm_eps=layer_norm_eps,
            hidden_dropout_prob=hidden_dropout_prob,
            intermediate_size=intermediate_size,
            hidden_act=hidden_act,
            hidden_size=hidden_size,
            key_query_shared_bottleneck=key_query_shared_bottleneck,
        )
        self.num_hidden_layers = num_hidden_layers
        self.pooler = (
            MobileBertPooler(
                classifier_activation=classifier_activation,
                hidden_size=hidden_size,
            )
            if add_pooling_layer
            else None
        )

        self.init_weights()

    def get_input_embeddings(self):
        return self.embeddings.word_embeddings

    def set_input_embeddings(self, value):
        self.embeddings.word_embeddings = value

    def get_head_mask(self, head_mask, num_hidden_layers, is_attention_chunked=False):
        """
        Prepare the head mask if needed.

        Args:
            head_mask (:obj:`paddle.Tensor` with shape :obj:`[num_heads]` or :obj:`[num_hidden_layers x num_heads]`, `optional`):
                The mask indicating if we should keep the heads or not (1.0 for keep, 0.0 for discard).
            num_hidden_layers (:obj:`int`):
                The number of hidden layers in the model.
            is_attention_chunked: (:obj:`bool`, `optional`, defaults to :obj:`False`):
                Whether or not the attentions scores are computed by chunks or not.

        Returns:
            :obj:`paddle.Tensor` with shape :obj:`[num_hidden_layers x batch x num_heads x seq_length x seq_length]` or
            list with :obj:`[None]` for each layer.
        """
        if head_mask is not None:
            head_mask = self._convert_head_mask_to_5d(head_mask, num_hidden_layers)
            if is_attention_chunked is True:
                head_mask = head_mask.unsqueeze(-1)
        else:
            head_mask = [None] * num_hidden_layers

        return head_mask

    def _convert_head_mask_to_5d(self, head_mask, num_hidden_layers):
        """-> [num_hidden_layers x batch x num_heads x seq_length x seq_length]"""
        if head_mask.dim() == 1:
            head_mask = head_mask.unsqueeze(0).unsqueeze(0).unsqueeze(-1).unsqueeze(-1)
            head_mask = head_mask.expand(num_hidden_layers, -1, -1, -1, -1)
        elif head_mask.dim() == 2:
            head_mask = head_mask.unsqueeze(1).unsqueeze(-1).unsqueeze(-1)  # We can specify head_mask for each layer
        assert head_mask.dim() == 5, f"head_mask.dim != 5, instead {head_mask.dim()}"
        head_mask = head_mask.to(dtype=self.dtype)  # switch to float if need + fp16 compatibility
        return head_mask

    def forward(
        self,
        input_ids,
        attention_mask=None,
        token_type_ids=None,
        position_ids=None,
        head_mask=None,
        inputs_embeds=None,
        output_hidden_states=None,
        output_attentions=None,
    ):
        r"""
        The MobileBertModel forward method, overrides the `__call__()` special method.

        Args:
            input_ids (Tensor):
                Indices of input sequence tokens in the vocabulary. They are
                numerical representations of tokens that build the input sequence.
                Its data type should be `int64` and it has a shape of [batch_size, sequence_length].
            token_type_ids (Tensor, optional):
                Segment token indices to indicate different portions of the inputs.
                Selected in the range ``[0, type_vocab_size - 1]``.
                If `type_vocab_size` is 2, which means the inputs have two portions.
                Indices can either be 0 or 1:
                - 0 corresponds to a *sentence A* token,
                - 1 corresponds to a *sentence B* token.
                Its data type should be `int64` and it has a shape of [batch_size, sequence_length].
                Defaults to `None`, which means we don't add segment embeddings.
            position_ids(Tensor, optional):
                Indices of positions of each input sequence tokens in the position embeddings. Selected in the range ``[0,
                max_position_embeddings - 1]``.
                Shape as `(batch_size, num_tokens)` and dtype as int64. Defaults to `None`.
            attention_mask (Tensor, optional):
                Mask used in multi-head attention to avoid performing attention on to some unwanted positions,
                usually the paddings or the subsequent positions.
                Its data type can be int, float and bool.
                When the data type is bool, the `masked` tokens have `False` values and the others have `True` values.
                When the data type is int, the `masked` tokens have `0` values and the others have `1` values.
                When the data type is float, the `masked` tokens have `-INF` values and the others have `0` values.
                It is a tensor with shape broadcasted to `[batch_size, num_attention_heads, sequence_length, sequence_length]`.
                Defaults to `None`, which means nothing needed to be prevented attention to.
            head_mask (:obj:`paddle.Tensor` with shape :obj:`[num_heads]` or :obj:`[num_hidden_layers x num_heads]`, `optional`):
                The mask indicating if we should keep the heads or not (1.0 for keep, 0.0 for discard). Defaults to `None`.
            output_hidden_states (bool, optional):
                Whether to return the output of each hidden layers.
                Defaults to `None`.
            output_attentions (bool, optional):
                Whether to return the output of each self attention layers.
                Defaults to `None`.

        Returns:
            tuple: Returns tuple (`sequence_output`, `pooled_output`) or (`encoder_outputs`, `pooled_output`).
            With the fields:
            - `sequence_output` (Tensor):
                Sequence of hidden-states at the last layer of the model.
                It's data type should be float32 and its shape is [batch_size, sequence_length, hidden_size].
            - `pooled_output` (Tensor):
                The output of first token (`[CLS]`) in sequence.
                We "pool" the model by simply taking the hidden state corresponding to the first token.
                Its data type should be float32 and its shape is [batch_size, hidden_size].
            - `encoder_outputs` (List(Tensor)):
                A list of Tensor containing hidden-states of the model at each hidden layer in the Transformer encoder.
                The length of the list is `num_hidden_layers`.
                Each Tensor has a data type of float32 and its shape is [batch_size, sequence_length, hidden_size].

        Example:
            .. code-block::
                import paddle
                from paddlenlp.transformers import MobileBertModel, MobileBertTokenizer
                tokenizer = MobileBertTokenizer.from_pretrained('mobilebert-uncased')
                model = MobileBertModel.from_pretrained('mobilebert-uncased')
                inputs = tokenizer("Welcome to use PaddlePaddle and PaddleNLP!")
                inputs = {k:paddle.to_tensor([v]) for (k, v) in inputs.items()}
                output = model(**inputs)
        """

        output_attentions = output_attentions is not None
        output_hidden_states = output_hidden_states is not None

        if input_ids is not None and inputs_embeds is not None:
            raise ValueError("You cannot specify both input_ids and inputs_embeds at the same time")
        elif input_ids is not None:
            input_shape = input_ids.shape
        elif inputs_embeds is not None:
            input_shape = inputs_embeds.shape[:-1]
        else:
            raise ValueError("You have to specify either input_ids or inputs_embeds")

        if attention_mask is None:
            attention_mask = paddle.ones(input_shape, dtype=input_ids.dtype)
        if token_type_ids is None:
            token_type_ids = paddle.zeros(input_shape, dtype="int64")

        # We can provide a self-attention mask of dimensions [batch_size, from_seq_length, to_seq_length]
        # ourselves in which case we just need to make it broadcastable to all heads.
        extended_attention_mask = attention_mask.unsqueeze(axis=[1, 2])
        extended_attention_mask = (1.0 - extended_attention_mask) * -10000.0

        # Prepare head mask if needed
        # 1.0 in head_mask indicate we keep the head
        # attention_probs has shape bsz x n_heads x N x N
        # input head_mask has shape [num_heads] or [num_hidden_layers x num_heads]
        # and head_mask is converted to shape [num_hidden_layers x batch x num_heads x seq_length x seq_length]
        head_mask = self.get_head_mask(head_mask, self.num_hidden_layers)

        embedding_output = self.embeddings(
            input_ids=input_ids, position_ids=position_ids, token_type_ids=token_type_ids, inputs_embeds=inputs_embeds
        )
        encoder_outputs = self.encoder(
            embedding_output,
            attention_mask=extended_attention_mask,
            head_mask=head_mask,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
        )
        sequence_output = encoder_outputs[0]
        pooled_output = self.pooler(sequence_output) if self.pooler is not None else None

        return (sequence_output, pooled_output) + encoder_outputs[1:]


class MobileBertForSequenceClassification(MobileBertPretrainedModel):
    """
    MobileBert Model with a linear layer on top of the output layer,
    designed for sequence classification/regression tasks like GLUE tasks.

    Args:
        mobilebert (:class:`MobileBertModel`):
            An instance of MobileBert.
        num_classes (int, optional):
            The number of classes. Defaults to `2`.
    """

    def __init__(self, mobilebert, num_labels=2):
        super(MobileBertForSequenceClassification, self).__init__()
        self.num_labels = num_labels
        self.mobilebert = mobilebert
        classifier_dropout = (
            self.mobilebert.config["classifier_dropout"]
            if self.mobilebert.config.get("classifier_dropout") is not None
            else self.mobilebert.config["hidden_dropout_prob"]
        )
        self.dropout = nn.Dropout(classifier_dropout)
        self.classifier = nn.Linear(self.mobilebert.config["hidden_size"], self.num_labels)

        self.init_weights()

    def forward(
        self,
        input_ids,
        attention_mask=None,
        token_type_ids=None,
        position_ids=None,
        head_mask=None,
        inputs_embeds=None,
        output_attentions=None,
        output_hidden_states=None,
    ):
        r"""
        The MobileBertForSequenceClassification forward method, overrides the __call__() special method.

        Args:
            input_ids (Tensor):
                See :class:`MobileBertModel`.
            token_type_ids (Tensor, optional):
                See :class:`MobileBertModel`.
            position_ids(Tensor, optional):
                See :class:`MobileBertModel`.
            head_mask (Tensor, optional):
                See :class:`MobileBertModel`.
            attention_mask (Tensor, optional):
                See :class:`MobileBertModel`.
            inputs_embeds (Tensor, optional):
                See :class:`MobileBertModel`.
            output_attentions (bool, optional):
                See :class:`MobileBertModel`.
            output_hidden_states (bool, optional):
                See :class:`MobileBertModel`.

        Returns:
            Tensor: Returns tensor `logits`, a tensor of the input text classification logits.
            Shape as `[batch_size, num_classes]` and dtype as float32.

        Example:
            .. code-block::
                import paddle
                from paddlenlp.transformers import MobileBertForSequenceClassification, MobileBertTokenizer
                tokenizer = MobileBertTokenizer.from_pretrained('mobilebert-uncased')
                model = MobileBertForSequenceClassification.from_pretrained('mobilebert-uncased', num_classes=2)
                inputs = tokenizer("Welcome to use PaddlePaddle and PaddleNLP!")
                inputs = {k:paddle.to_tensor([v]) for (k, v) in inputs.items()}
                logits = model(**inputs)
                print(logits.shape)
                # [1, 2]
        """

        outputs = self.mobilebert(
            input_ids,
            attention_mask=attention_mask,
            token_type_ids=token_type_ids,
            position_ids=position_ids,
            head_mask=head_mask,
            inputs_embeds=inputs_embeds,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
        )

        pooled_output = outputs[1]

        pooled_output = self.dropout(pooled_output)
        logits = self.classifier(pooled_output)

        return logits


class MobileBertForQuestionAnswering(MobileBertPretrainedModel):
    """
    MobileBert Model with a linear layer on top of the hidden-states output to compute `span_start_logits`
    and `span_end_logits`, designed for question-answering tasks like SQuAD.

    Args:
        mobilebert (:class:`MobileBert`):
            An instance of MobileBert.
    """

    def __init__(self, mobilebert):
        super(MobileBertForQuestionAnswering, self).__init__()
        self.num_labels = 2
        self.mobilebert = mobilebert
        self.qa_outputs = nn.Linear(self.mobilebert.config["hidden_size"], self.num_labels)

        self.init_weights()

    def forward(
        self,
        input_ids,
        attention_mask=None,
        token_type_ids=None,
        position_ids=None,
        head_mask=None,
        inputs_embeds=None,
        start_positions=None,
        end_positions=None,
        output_attentions=None,
        output_hidden_states=None,
    ):
        r"""
        The MobileBertForQuestionAnswering forward method, overrides the __call__() special method.

        Args:
            input_ids (Tensor):
                See :class:`MobileBertModel`.
            token_type_ids (Tensor, optional):
                See :class:`MobileBertModel`.
            position_ids(Tensor, optional):
                See :class:`MobileBertModel`.
            head_mask (Tensor, optional):
                See :class:`MobileBertModel`.
            attention_mask (Tensor, optional):
                See :class:`MobileBertModel`.
            inputs_embeds (Tensor, optional):
                See :class:`MobileBertModel`.
            output_attentions (bool, optional):
                See :class:`MobileBertModel`.
            output_hidden_states (bool, optional):
                See :class:`MobileBertModel`.
            start_positions (Tensor, optional):
                Labels for position (index) of the start of the labelled span for computing the token classification loss.
                Positions are clamped to the length of the sequence (:obj:`sequence_length`). Position outside of the
                sequence are not taken into account for computing the loss.
            end_positions (Tensor, optional):
                Labels for position (index) of the end of the labelled span for computing the token classification loss.
                Positions are clamped to the length of the sequence (:obj:`sequence_length`). Position outside of the
                sequence are not taken into account for computing the loss.

        Returns:
            tuple: Returns tuple (`start_logits`, `end_logits`).
            With the fields:
            - `start_logits` (Tensor):
                A tensor of the input token classification logits, indicates the start position of the labelled span.
                Its data type should be float32 and its shape is [batch_size, sequence_length].
            - `end_logits` (Tensor):
                A tensor of the input token classification logits, indicates the end position of the labelled span.
                Its data type should be float32 and its shape is [batch_size, sequence_length].

        Example:
            .. code-block::
                import paddle
                from paddlenlp.transformers import MobileBertForQuestionAnswering, MobileBertTokenizer
                tokenizer = MobileBertTokenizer.from_pretrained('mobilebert-uncased')
                model = MobileBertForQuestionAnswering.from_pretrained('mobilebert-uncased')
                inputs = tokenizer("Welcome to use PaddlePaddle and PaddleNLP!")
                inputs = {k:paddle.to_tensor([v]) for (k, v) in inputs.items()}
                outputs = model(**inputs)
                start_logits = outputs[0]
                end_logits = outputs[1]
        """
        outputs = self.mobilebert(
            input_ids,
            attention_mask=attention_mask,
            token_type_ids=token_type_ids,
            position_ids=position_ids,
            head_mask=head_mask,
            inputs_embeds=inputs_embeds,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
        )

        sequence_output = outputs[0]
        logits = self.qa_outputs(sequence_output)

        logits = paddle.transpose(logits, perm=[2, 0, 1])

        start_logits, end_logits = paddle.unstack(x=logits, axis=0)

        output = (start_logits, end_logits) + outputs[2:]

        return output



# coding:utf-8
# Copyright (c) 2022  PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys
import errno
import json
import math
import threading
from multiprocessing import cpu_count
import paddle
from paddle import inference
from ..transformers import PretrainedModel
from ..utils.log import logger
from ..taskflow.utils import dygraph_mode_guard


class Predictor:
    def __init__(self, model_path, precision, device):
        self._model_path = model_path
        self._default_static_model_path = "auto_static"
        self._precision = precision
        self._cpu_thread = 8
        self._config = None
        self._device = device
        self._num_threads = math.ceil(cpu_count() / 2)
        self._output_num = 1
        paddle.set_device(device)
        self._create_predictor()
        self._lock = threading.Lock()

    def _get_default_static_model_path(self):
        # The model path had the static_model_path
        static_model_path = os.path.join(self._model_path, self._default_static_model_path, "inference.pdmodel")
        if os.path.exists(static_model_path):
            return os.path.join(self._model_path, self._default_static_model_path, "inference")
        for file_name in os.listdir(self._model_path):
            # FIXME(wawltor) The path maybe not correct
            if file_name.count(".pdmodel"):
                return os.path.join(self._model_path, file_name[:-8])
        return None

    def _is_int8_model(self, model_path):
        paddle.set_device("cpu")
        model = paddle.jit.load(model_path)
        program = model.program()
        for block in program.blocks:
            for i, op in enumerate(block.ops):
                if op.type.count("quantize"):
                    paddle.set_device(self._device)
                    return True
        paddle.set_device(self._device)
        return False

    def _create_predictor(self):
        # Get the model parameter path and model config path
        static_model_path = self._get_default_static_model_path()

        # Convert the Draph Model to Static Model
        is_from_static = True
        if static_model_path is None:
            raise RuntimeError("The model path do not include the inference model, please check!")
        is_int8_model = self._is_int8_model(static_model_path)
        # Load the inference model and maybe we will convert the onnx model
        # Judge the predictor type for the inference
        if self._precision == "int8" and not is_int8_model:
            self._precision = "fp32"

        if is_int8_model:
            self._precision = "int8"

        self._predictor_type = self._check_predictor_type()
        if self._predictor_type == "paddle_inference":
            self._prepare_paddle_mode(static_model_path)
        else:
            self._prepare_onnx_mode(static_model_path)

    def _check_predictor_type(self):
        predictor_type = "paddle_inference"
        device = paddle.get_device()
        if self._precision == "int8" or device == "xpu" or device == "cpu":
            predictor_type = "paddle_inference"
        else:
            if device.count("gpu") and self._precision == "fp16":
                try:
                    import onnx
                    import onnxruntime as ort
                    import paddle2onnx
                    from onnxconverter_common import float16

                    predictor_type = "onnxruntime"
                except:
                    logger.error(
                        "The inference precision is change to 'fp32', please install the dependencies that required for 'fp16' inference, you could use the commands as fololws:\n"
                        " ****** pip uninstall onnxruntime ******\n"
                        " ****** pip install onnxruntime-gpu onnx onnxconverter-common ******"
                    )
                    sys.exit(-1)
        return predictor_type

    def _prepare_paddle_mode(self, static_model_path):
        """
        Construct the input data and predictor in the PaddlePaddele static mode.
        """
        self._config = paddle.inference.Config(static_model_path + ".pdmodel", static_model_path + ".pdiparams")
        self._config.disable_glog_info()
        if paddle.get_device() == "cpu":
            self._config.disable_gpu()
            self._config.enable_mkldnn()
            self._config.enable_memory_optim()
            if self._precision == "int8":
                config.enable_mkldnn_bfloat16()
            elif self._precision == "fp16":
                config.enable_mkldnn_int8()
        else:
            self._config.enable_use_gpu(100, int(self._device.split(":")[-1]))
            precision_type = inference.PrecisionType.Float32
            if self._precision == "int8":
                precision_type = inference.PrecisionType.INT8
                # FIXME(wawltor) The paddlenlp serving support the int8 model
                logger.warning("The PaddleNLP serving do not support the INT8 model, we will support later!")
                sys.exit(-1)

        self._config.switch_use_feed_fetch_ops(False)
        self._config.set_cpu_math_library_num_threads(self._num_threads)
        self._config.delete_pass("embedding_eltwise_layernorm_fuse_pass")
        self._predictor = paddle.inference.create_predictor(self._config)
        self._input_handles = [self._predictor.get_input_handle(name) for name in self._predictor.get_input_names()]
        self._output_handles = [self._predictor.get_output_handle(name) for name in self._predictor.get_output_names()]
        self._output_num = len(self._output_handles)

    def _prepare_onnx_mode(self, static_model_path):
        import onnx
        import onnxruntime as ort
        import paddle2onnx
        from onnxconverter_common import float16

        onnx_dir = os.path.join(self._model_path, "onnx")
        if not os.path.exists(onnx_dir):
            os.mkdir(onnx_dir)
        float_onnx_file = os.path.join(onnx_dir, "model.onnx")
        if not os.path.exists(float_onnx_file):
            model_path = static_model_path + ".pdmodel"
            params_file = static_model_path + ".pdiparams"
            onnx_model = paddle2onnx.command.c_paddle_to_onnx(
                model_file=model_path, params_file=params_file, opset_version=13, enable_onnx_checker=True
            )
            with open(float_onnx_file, "wb") as f:
                f.write(onnx_model)
        fp16_model_file = os.path.join(onnx_dir, "fp16_model.onnx")
        if not os.path.exists(fp16_model_file):
            onnx_model = onnx.load_model(float_onnx_file)
            trans_model = float16.convert_float_to_float16(onnx_model, keep_io_types=True)
            onnx.save_model(trans_model, fp16_model_file)
        providers = ["CUDAExecutionProvider"]
        sess_options = ort.SessionOptions()
        sess_options.inter_op_num_threads = self._num_threads
        device_id = int(self._device.split(":")[-1])
        self._predictor = ort.InferenceSession(
            fp16_model_file,
            sess_options=sess_options,
            providers=providers,
            provider_options=[{"device_id": device_id}],
        )
        self._output_num = len(self._predictor.get_outputs())
        assert "CUDAExecutionProvider" in self._predictor.get_providers(), (
            f"The environment for GPU inference is not set properly. "
            "A possible cause is that you had installed both onnxruntime and onnxruntime-gpu. "
            "Please run the following commands to reinstall: \n "
            "1) pip uninstall -y onnxruntime onnxruntime-gpu \n 2) pip install onnxruntime-gpu"
        )

    def _convert_dygraph_to_static(self, model_instance, input_spec):
        """
        Convert the dygraph model to static model.
        """
        assert (
            model_instance is not None
        ), "The dygraph model must be created before converting the dygraph model to static model."
        assert (
            input_spec is not None
        ), "The input spec must be created before converting the dygraph model to static model."
        logger.info(
            "Converting to the static inference model will cost a little time, please do not break this process."
        )
        try:
            static_model = paddle.jit.to_static(model_instance, input_spec=input_spec)
            save_path = os.path.join(self._model_path, self._default_static_model_path, "inference")
            paddle.jit.save(static_model, save_path)
            logger.info("The static inference model save in the path:{}".format(save_path))
        except:
            logger.warning(
                "Fail convert to inference model, please create the issue for the developers,"
                "the issue link: https://github.com/PaddlePaddle/PaddleNLP/issues"
            )
            sys.exit(-1)



# coding:utf-8
# Copyright (c) 2022  PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import numpy as np

from .base_handler import BaseModelHandler


class QAModelHandler(BaseModelHandler):
    def __init__(self):
        super().__init__()

    @classmethod
    def process(cls, predictor, tokenizer, data, parameters):

        max_seq_len = 128
        doc_stride = 128
        batch_size = 1
        if "max_seq_len" in parameters:
            max_seq_len = parameters["max_seq_len"]
        if "batch_size" in parameters:
            batch_size = parameters["batch_size"]
        if "doc_stride" in parameters:
            doc_stride = parameters["doc_stride"]
        context = None
        question = None

        # Get the context in qa task
        if "context" in data:
            context = data["context"]
        if context is None:
            return {}
        if isinstance(context, str):
            context = [context]

        # Get the context in qa task
        if "question" in data:
            question = data["question"]
        if question is None:
            return {}
        if isinstance(question, str):
            question = [question]

        tokenizer_results = tokenizer(
            question,
            context,
            stride=doc_stride,
            max_length=max_seq_len,
            return_offsets_mapping=True,
            pad_to_max_seq_len=True,
        )
        input_ids = tokenizer_results["input_ids"]
        token_type_ids = tokenizer_results["token_type_ids"]
        # Seperates data into some batches.
        batches = [[i, i + batch_size] for i in range(0, len(input_ids), batch_size)]

        results = [[] for i in range(0, predictor._output_num)]
        for start, end in batches:
            input_id = np.array(input_ids[start:end]).astype("int64")
            token_type_id = np.array(token_type_ids[start:end]).astype("int64")
            if predictor._predictor_type == "paddle_inference":
                predictor._input_handles[0].copy_from_cpu(input_id)
                predictor._input_handles[1].copy_from_cpu(token_type_id)

                predictor._predictor.run()
                output = [output_handle.copy_to_cpu() for output_handle in predictor._output_handles]
                for i, out in enumerate(output):
                    results[i].extend(out.tolist())
            else:
                output = predictor._predictor.run(None, {"input_ids": input_id, "token_type_ids": token_type_id})
                for i, out in enumerate(output):
                    results[i].extend(out.tolist())
        data["offset_mapping"] = tokenizer_results["offset_mapping"]
        out_dict = {"logits": results[0], "data": data}
        for i in range(1, len(results)):
            out_dict[f"logits_{i}"] = results[1]
        return out_dict



# Copyright (c) 2021 PaddlePaddle Authors. All Rights Reserved.
# Copyright 2018 The Google AI Language Team Authors and The HuggingFace Inc. team.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import pickle
import six
import shutil

from paddle.utils import try_import
from paddlenlp.utils.env import MODEL_HOME

from ..ernie.tokenizer import ErnieTokenizer

__all__ = ["ErnieGramTokenizer"]


class ErnieGramTokenizer(ErnieTokenizer):
    r"""
    Constructs an ERNIE-Gram tokenizer. It uses a basic tokenizer to do punctuation splitting, lower casing and so on,
    and follows a WordPiece tokenizer to tokenize as subwords.

    This tokenizer inherits from :class:`~paddlenlp.transformers.ernie.tokenizer.ErnieTokenizer`.
    For more information regarding those methods, please refer to this superclass.

    Args:
        vocab_file (str):
            The vocabulary file path (ends with '.txt') required to instantiate
            a `WordpieceTokenizer`.
        do_lower_case (str, optional):
            Whether or not to lowercase the input when tokenizing.
            Defaults to `True`.
        unk_token (str, optional):
            A special token representing the *unknown (out-of-vocabulary)* token.
            An unknown token is set to be `unk_token` inorder to be converted to an ID.
            Defaults to "[UNK]".
        sep_token (str, optional):
            A special token separating two different sentences in the same input.
            Defaults to "[SEP]".
        pad_token (str, optional):
            A special token used to make arrays of tokens the same size for batching purposes.
            Defaults to "[PAD]".
        cls_token (str, optional):
            A special token used for sequence classification. It is the last token
            of the sequence when built with special tokens. Defaults to "[CLS]".
        mask_token (str, optional):
            A special token representing a masked token. This is the token used
            in the masked language modeling task which the model tries to predict the original unmasked ones.
            Defaults to "[MASK]".

    Examples:
        .. code-block::

            from paddlenlp.transformers import ErnieGramTokenizer
            tokenizer = ErnieGramTokenizer.from_pretrained('ernie-gram-zh')
            encoded_inputs = tokenizer('He was a puppeteer')
            # encoded_inputs:
            # {
            #   'input_ids': [1, 4444, 4385, 1545, 6712, 10062, 9568, 9756, 9500, 2],
            #   'token_type_ids': [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
            # }

    """
    resource_files_names = {"vocab_file": "vocab.txt"}  # for save_pretrained
    pretrained_resource_files_map = {
        "vocab_file": {
            "ernie-gram-zh": "https://bj.bcebos.com/paddlenlp/models/transformers/ernie_gram_zh/vocab.txt",
            "ernie-gram-zh-finetuned-dureader-robust": "https://bj.bcebos.com/paddlenlp/models/transformers/ernie_gram_zh/vocab.txt",
        }
    }
    pretrained_init_configuration = {
        "ernie-gram-zh": {"do_lower_case": True},
        "ernie-gram-zh-finetuned-dureader-robust": {"do_lower_case": True},
    }
    max_model_input_sizes = {
        "ernie-gram-zh": 512,
        "ernie-gram-zh-finetuned-dureader-robust": 512,
    }

    def __init__(
        self,
        vocab_file,
        do_lower_case=True,
        unk_token="[UNK]",
        sep_token="[SEP]",
        pad_token="[PAD]",
        cls_token="[CLS]",
        mask_token="[MASK]",
        **kwargs
    ):
        super(ErnieGramTokenizer, self).__init__(
            vocab_file,
            do_lower_case=do_lower_case,
            unk_token=unk_token,
            sep_token=sep_token,
            pad_token=pad_token,
            cls_token=cls_token,
            mask_token=mask_token,
            **kwargs,
        )
